/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";

var anonymox = {};

Components.utils.import("resource://anonymox/components.jsm", anonymox);

anonymox.overlay = {
		
	
	installButton : function(toolbarId, id, afterId) {
	    if (!document.getElementById(id)) {
	        var toolbar = document.getElementById(toolbarId);

	        
	        var before = null;
	        if (afterId) {
	            let elem = document.getElementById(afterId);
	            if (elem && elem.parentNode == toolbar)
	                before = elem.nextElementSibling;
	        }

	        toolbar.insertItem(id, before);
	        toolbar.setAttribute("currentset", toolbar.currentSet);
	        document.persist(toolbar.id, "currentset");

	        if (toolbarId == "addon-bar")
	            toolbar.collapsed = false;
	    }
	},

	
	progressListener : {
		QueryInterface : function(aIID) {
			if (aIID.equals(Ci.nsIWebProgressListener)
					|| aIID.equals(Ci.nsISupportsWeakReference)
					|| aIID.equals(Ci.nsISupports))
				return this;
			throw Components.results.NS_NOINTERFACE;
		},
		
		onLocationChange: function(aWebProgress, aRequest, aURI) {
			let info = new anonymox.Info("overlay.progressListener.onLocationChange");
			try {
				let browser = gBrowser.getBrowserForDocument(aWebProgress.DOMWindow.document);
				info.verbose("spec: " + aURI.spec + " DOMWindow " + aWebProgress.DOMWindow + " browser " + browser);
				if (browser) {
					anonymox.ui.locationOrProfileChange(window, browser, aURI.spec);
				} else {
					info.debug("ABORT: No browser found for " + aURI);
				}
			} catch(e) {
				info.error(e);
			}
		},
		
		onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus) {
			let info = new anonymox.Info("overlay.progressListener.onStateChange");
			try {
				info.verbose(new Date().getTime() + " " + aStateFlags.toString(16) + "|" + aStatus.toString(16));
				
				if (aStateFlags & Ci.nsIWebProgressListener.STATE_START) {
					
					if (aRequest) {
						
						if (aRequest instanceof Ci.nsIChannel || "URI" in aRequest) {
		
							info.verbose("Started: " + aRequest.URI.spec);
							
							let browser = gBrowser.getBrowserForDocument(aWebProgress.DOMWindow.document);
							if (browser) {
								anonymox.ui.locationOrProfileChange(window, browser, aRequest.URI.spec);
							} else {
								info.verbose("ABORT: No browser found for " + aRequest.URI.spec);
							}
						} else {
							info.verbose("ABORT: Cannot get URI from aRequest");
						}
					} else {
						info.verbose("ABORT: No aRequest");
					}
				} else if (aStateFlags & Ci.nsIWebProgressListener.STATE_STOP) {
					
					
		            
					
					info.verbose("STATE_STOP status " + aStatus);
	
					if (aStatus == Components.results.NS_ERROR_PROXY_CONNECTION_REFUSED) {
						info.warning("Detected possible Proxy-Problem.(NS_ERROR_PROXY_CONNECTION_REFUSED)");
						
					}
					
					try {
						
						let browser = gBrowser.getBrowserForDocument(aWebProgress.DOMWindow.document);
						if (browser) {
							anonymox.ui.locationOrProfileChange(window, browser);
						} else {
							info.verbose("ABORT: No browser found");
						}
					} catch(e) {
						
					}
				}
			} catch(e) {
				info.error(e);
			}
		}
	}
};



window.addEventListener("load", function() {
	if (!anonymox.state.isLoaded)
		anonymox.core.startUp(window);
	else
		anonymox.ui.newWindowOpened(window);
}, false);
document.addEventListener("anonymox_websiteComm", function(e) { anonymox.core.websiteCommEventHandler(e); }, false, true);
