/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";

Components.utils.import("resource://anonymox/components.jsm", anonymox);

anonymox.panel = {
	
	lastOrigin : undefined,
	
	
	open : function(origin) {
		let info = new anonymox.Info("panel.open");
		try {
			anonymox.ui.panelWindow = window;
			let panelElement = document.getElementById("anonymox-panel");

			if (!origin)	
				origin = anonymox.panel.lastOrigin;
			
			if (!anonymox.state.isReady) {
				if (anonymox.state.isStarting) {
					if (origin == "networkStatus")
						return;
					else
						anonymox.ui.notification(1, "loadingPleaseWait");
				} else {
					anonymox.core.switchOn();
				}
				return;
			}
			if (window.anonymoxdata.profile.isInternal) {
				if (origin == "networkStatus") {
					return;
				} else {
					anonymox.ui.notification(0, "panelNotForThisSite", undefined, undefined, window);
					return;
				}
			}
			
			
			document.getElementById('anonymox-panel-status-container').hidden = anonymox.state.displayStatusURI == undefined;
			
			
			document.getElementById('anonymox-panel-listscontainer').hidden = !anonymox.prefs.getBoolPref("displayProfileBar");
			
			
			document.getElementById("anonymox-panel-cookieaction-menu").selectedIndex = anonymoxdata.profile.cookieAction;
			
			
				
			info.sendMsg(anonymox.utils.msgType.usage, "panel", "open|" + origin);

			if ( origin == "toolbarButton" || origin == "mainMenu" || origin == "toolsMenu" || origin == "networkStatus")
				panelElement.openPopup(document.getElementById('anonymox-button'), 'after_start', 0, 0, false, false);
			else if (origin == "statusBar")
				panelElement.openPopup(document.getElementById('anonymox-statusBarIcon'), 'before_start', 0, 0, false, false);
			else
				info.error("Unknown origin: '" + origin + "'");

			anonymox.panel.lastOrigin = origin;
		} catch(e) {
			info.error(e);
		}
	},
	
	profileChangedHook : function() {
		let panelElement = document.getElementById("anonymox-panel");
		
		if (panelElement.state == "open")
			anonymox.panel.rebuildAll();	
	},
	
	onPopupShowingHandler : function() {
		let info = new anonymox.Info("panel.onPopupShowingHandler");
		
		let panelElement = document.getElementById("anonymox-panel");

		anonymox.panel.showValidity(true);
		
		if (panelElement.state != "open")
			anonymox.panel.rebuildAll();
	},
	
	rebuildSiteList : function() {
		let info = new anonymox.Info("rebuildSiteList");
		
		let siteList = document.getElementById('anonymox-panel-pickservice-site'),
			menuPopup = siteList.firstChild;

		
		while (menuPopup.hasChildNodes()) {
			menuPopup.removeChild(menuPopup.lastChild);
		}
		
		
		
		let n = 0;
		for ( let i = 0; i < anonymox.profilemgr.list.length; i++) {
			let profile = anonymox.profilemgr.list[i];
			if (!profile.isInternal && profile.site != "*")
				n++;
		}
		
		let itemAll;
		if (n == 0)
			itemAll = siteList.appendItem(anonymox.localize("panelAll"), "*");
		else
			itemAll = siteList.appendItem(anonymox.localize("panelAllWithExceptions", n), "*");
		
		if (!window.anonymoxdata.uri)
			info.error("rebuildSiteList: window.anonymoxdata.uri is not set!");

		let filters = anonymox.utils.getDomainPartsFromURI(window.anonymoxdata.uri);
		let itemSpec;
		if (filters != null) {
			let domain = filters[filters.length - 2];	
			itemSpec = siteList.appendItem(anonymox.profilemgr.getReadableName(domain), domain);
		}
		
		
		if (anonymox.profilemgr.getProfileForURI(window.anonymoxdata.uri).site != "*") {
			if (itemSpec)
				siteList.selectedItem = itemSpec;
			else
				siteList.selectedItem = itemAll;
		} else {
			siteList.selectedItem = itemAll;
		}
		
	},
	
	rebuildAll : function() {
		anonymox.panel.rebuildActiveCheckBoxAndContainers();
		anonymox.panel.rebuildSiteList();
		anonymox.panel.rebuildSiteOption();
		
		let identButton = document.getElementById('anonymox-panel-main-identity-button');
		identButton.hidden = !anonymoxdata.profile.gatewayFilter.enabled;

		if (window.anonymoxdata.profile.gatewayFilter.enabled) {
			anonymox.panel.rebuildIdentitiesList();
			anonymox.panel.rebuildCountryList();
		}
		
		anonymox.panel.rebuildSiteOption();
		anonymox.panel.rebuildAttrib(undefined);
		
		anonymox.panel.rebuildProfilesList();
		anonymox.panel.rebuildServicesList();

		anonymox.panel.buildStaticElements();	
		
		anonymox.panel.apply(true);
	},
	
	identClickFuncMaker : function(gw) {
		return function() {
			let info = new anonymox.Info("identClickFuncMaker");
			window.anonymoxdata.profile.wantGateway(gw);
			info.sendMsg(anonymox.utils.msgType.usage, "features", "selectGateway");
		};
	},
	
	rebuildIdentitiesList : function(invalidFilter) {
		let info = new anonymox.Info("panel.rebuildIdentitiesList");
		info.verbose("");
		let identList = document.getElementById('anonymox-panel-main-identity-list');
		
		while (identList.firstChild) {
			identList.removeChild(identList.firstChild);
		}

		if (invalidFilter) {
			let newItem = identList.appendItem(anonymox.localize("panelNoGWAvailable"));
			newItem.setAttribute("style", "background-color:red;");
			document.getElementById('anonymox-panel-main-identity-list-label').value = anonymox.localize('panelIdentitiesLabel') + ' (0)';
			return;
		}
		
		document.getElementById('anonymox-panel-main-identity-list-label').value = anonymox.localize('panelIdentitiesLabel') + ' (' + window.anonymoxdata.profile.gatewayRotateList.length + ')';
		
		for ( let i = 0; i < window.anonymoxdata.profile.gatewayRotateList.length; i++ ) {
			let gw = window.anonymoxdata.profile.gatewayRotateList[i];
			let newItem = identList.appendItem(gw.id, gw);
			
			
			
			let priBox = document.createElement("hbox");
			priBox.setAttribute("flex", "0");
			newItem.appendChild(priBox);
			
			anonymox.config.attributes.forEach(function(attrib) {
				if (gw.attributes[attrib]) {
					let icon = document.createElement("image");
					icon.setAttribute("src", "chrome://anonymox/skin/attributes/" + attrib + "16.png");
					icon.setAttribute("tooltiptext", anonymox.localize("attrib_" + attrib));
					priBox.appendChild(icon);
				}
			});
			
			let secBox = document.createElement("hbox");
			secBox.setAttribute("flex", "6000000");
			newItem.appendChild(secBox);
			
			if (window.anonymoxdata.profile.gatewayRotateList.length > 1)
				newItem.addEventListener("click", anonymox.panel.identClickFuncMaker(gw), false);
			
			if (gw == window.anonymoxdata.profile.gateway) {
				identList.selectedIndex = i;
				identList.ensureIndexIsVisible(i);
			}
		}
	},
	
	rebuildActiveCheckBoxAndContainers : function() {
		let activeCB = document.getElementById('anonymox-panel-active');
		let scdLvlCont = document.getElementById('anonymox-panel-2ndLevelcontainer');
		let cookiesCont = document.getElementById('anonymox-panel-cookieActionContainer');
		
		let isOn = window.anonymoxdata.profile.gatewayFilter.enabled;
		
		activeCB.style.color = isOn ? "green":"red";
		activeCB.checked = isOn;
		scdLvlCont.hidden = !isOn;
		cookiesCont.hidden = !isOn;
	},
	
	rebuildAttrib : function(attrib, win) {
		if (!attrib) {
			anonymox.panel.rebuildAttrib("stealth", win);
			anonymox.panel.rebuildAttrib("fast", win);
			anonymox.panel.rebuildAttrib("premium", win);
		} else {
			let attribCheck = document.getElementById('anonymox-panel-attributes-' + attrib);
			attribCheck.setAttribute("label", anonymox.localize("attrib_" + attrib));
			attribCheck.checked = window.anonymoxdata.profile.gatewayFilter.attributes[attrib];
		}
	},

	rebuildSiteOption : function() {
		let info = new anonymox.Info("panel.rebuildSiteOption");
		try {
			let siteList = document.getElementById('anonymox-panel-pickservice-site');
			let warningLabel = document.getElementById('anonymox-panel-pickservice-profilewarning');

			
			let n = 0;
			for ( let i = 0; i < anonymox.profilemgr.list.length; i++ ) {
				let profile = anonymox.profilemgr.list[i];
				if (!profile.isInternal && profile.site != "*")
					n++;
			}
			
			
			warningLabel.hidden = siteList.selectedItem.value == "*";
			
		} catch(e) {
			info.error(e);
		}
	},

	CustomMenuitem: function (menuitem) {
		
		

		let info = new anonymox.Info("panel.customMenuitem"),

			
			menupopup = menuitem.getElementsByTagName("menupopup")[0],
			items = [];

		return {
			removeAllItems: function () {
				info.verbose("removeAllItems()");
				while (menupopup.hasChildNodes()) {
					menupopup.removeChild(menupopup.firstChild);
				}

				
				items = [];
			},

			addMenuitem: function (labelStr, value, imagePath) {
				let mi = document.createElement("menuitem"),
					img = document.createElement("image"),
					label = document.createElement("label");

				mi.setAttribute("label", labelStr);
				mi.setAttribute("value", value);
				mi.setAttribute("image", imagePath);

				img.setAttribute("src", imagePath);
				img.setAttribute("style", "vertical-align: middle;");

				label.setAttribute("value", labelStr);

				mi.appendChild(img);
				mi.appendChild(label);

				menupopup.appendChild(mi);

				
				if (items.length === 0) {
					info.verbose("set <menuitem> as slectedItem");
					menuitem.selectedItem = mi;
				}

				items.push(mi);

				return items.length - 1;
			},

			setSelectedItem: function (index) {
				menuitem.selectedItem = items[index];
			},

			getElement: function (index) {
				return items[index];
			}
		};
	},
	
	rebuildCountryList : function () {
		let info = new anonymox.Info("panel.rebuildCountryList");
		
		let index = 0,
			countryListElement = document.getElementById("anonymox-panel-pickservice-country"),
			countries = {},
			countryList = anonymox.panel.CustomMenuitem(countryListElement),
			flagPath = function (flagName) {
				return "chrome://anonymox/skin/flags/" + flagName + ".png";
			};

		
		countryList.removeAllItems();

		
		countryList.addMenuitem(anonymox.localize("panelAll"), "all", flagPath("world"));

		
		for (let gwi = 0; gwi < anonymox.core.gatewayListCache.length; gwi++) {
			let gw = anonymox.core.gatewayListCache[gwi];
			
			
			if (!countries[gw.country]) {
				index = countryList.addMenuitem(gw.country.toUpperCase(), gw.country, flagPath(gw.country));
				countries[gw.country] = true;
				
				if (window.anonymoxdata.profile != undefined && window.anonymoxdata.profile.gatewayFilter != undefined && window.anonymoxdata.profile.gatewayFilter.country == gw.country) {
					countryList.setSelectedItem(index);
				}
			}
		}

		if (anonymox.ui.showMoreCountries) {
			index = countryList.addMenuitem(anonymox.localize("panelMoreCountries"), "anonymox-more-countries", flagPath("../attributes/premium16"));
			countryList.getElement(index).setAttribute("id", "anonymox-more-countries");
		}
	},

	showValidity : function(isValid) {
		["country","attributes"].forEach(function(name) {
			document.getElementById('anonymox-panel-pickservice-' + name + '-label').setAttribute("class", "anonymox-pickservice-categorylabel" + (isValid?"":"_red"));
		});
	},
	
	activeButtonClick : function() {
		let info = new anonymox.Info("panel.activeButtonClick");
		
		let activeCB = document.getElementById('anonymox-panel-active');
		let site = document.getElementById('anonymox-panel-pickservice-site').selectedItem.value;
		
		let curFilter = anonymoxdata.profile.gatewayFilter;
		if (!curFilter.enabled && activeCB.checked) {
			
			curFilter.enabled = true;
			anonymox.profilemgr.setProfile(curFilter, site);
			anonymox.panel.rebuildAll();
		} else if (curFilter.enabled && !activeCB.checked) {
			
			curFilter.enabled = false;
			anonymox.profilemgr.setProfile(curFilter, site);
			anonymox.panel.rebuildAll();
		}
	},
	
	apply : function(dryRun) {
		let info = new anonymox.Info("panel.apply");
		try {
			let site = document.getElementById('anonymox-panel-pickservice-site').selectedItem.value;
			let activeCB = document.getElementById('anonymox-panel-active');
			
			let filter = {
				enabled : activeCB.checked ? true : false,
				attributes : {}
			};
			
			if (filter.enabled) {
				let countryList = document.getElementById('anonymox-panel-pickservice-country');
				let cListValue = countryList.selectedItem.getAttribute("value");
				filter.country = cListValue != "all" ? cListValue : undefined;
			}
			
			anonymox.config.attributes.forEach(function(attrib) {
				if (document.getElementById('anonymox-panel-attributes-' + attrib).checked)
					filter.attributes[attrib] = true;
			});
			
			
			if (anonymox.profilemgr.filterYieldsResults(filter, site)) {
				if (!dryRun) {
					
					anonymox.profilemgr.setProfile(filter, site);

					anonymox.panel.showValidity(true);
					
					anonymox.panel.rebuildAll();
				
					info.sendMsg(anonymox.utils.msgType.usage, "panel", "apply|" + site + "|" + filter.enabled + "|" + anonymox.profilemgr.getCountryName(filter.country));
				}
			} else {
				info.notice("Current filter is invalid");
				anonymox.panel.showValidity(false);
				
				
				if (!window.anonymoxdata.profile.gatewayFilter.enabled && filter.enabled) {
					info.notice("invalid filter, but want to start -> start with standard");
					anonymox.profilemgr.setProfile(anonymox.profilemgr.defaultFilter, site);
					anonymox.panel.rebuildActiveCheckBoxAndContainers();
				}
				
				anonymox.panel.rebuildIdentitiesList(true);
			}
		} catch(e) {
			info.error(e);
		}
	},
	
	buildStaticElements : function() {
		let tooltipText = "";
		anonymox.config.attributes.forEach(function(attrib) {
			tooltipText += anonymox.localize("attrib_" + attrib) + " - " + anonymox.localize("attrib_" + attrib + "_desc") + "\n";
			document.getElementById("anonymox-panel-attributes-" + attrib).setAttribute("tooltiptext", anonymox.localize("attrib_" + attrib + "_desc"));
		});
		document.getElementById("anonymox-panel-pickservice-attributes-helpicon").setAttribute("tooltiptext", tooltipText);
	},

	
	rebuildServicesList : function() {
		let serviceList = document.getElementById("anonymox-panel-gatewaylist");
		
		
		for ( let i = serviceList.getRowCount() - 1; i >= 0; i--) {
			serviceList.removeItemAt(0);
		}
		
		let gw,
			tmpRowList = [];
		
		for (let i = 0; i < anonymox.core.gatewayListCache.length + 1; i++) {
			if (i != anonymox.core.gatewayListCache.length)
				gw = anonymox.core.gatewayListCache[i];
			else
				gw = anonymox.profilemgr.nullGateway;
			
			let row = document.createElement('listitem');
			row.value = gw.id;
			
			let cell;
			cell = document.createElement('listcell');
			cell.setAttribute('label', gw.id);
			row.appendChild(cell);

			cell = document.createElement('listcell');
			cell.setAttribute('label', gw.ip ? (gw.ip + ":" + gw.port) : "n/a");
			row.appendChild(cell);
			
			cell = document.createElement('listcell');
			cell.setAttribute('label', gw.isOnline ? "yes" : (gw.isOnline == undefined ? "check.." : "no") );
			row.appendChild(cell);
			
			cell = document.createElement('listcell');
			cell.setAttribute('label', gw.usageIndex ? gw.usageIndex + "%":"n/a");
			row.appendChild(cell);
			
			cell = document.createElement('listcell');
			cell.setAttribute('label', gw.lastLatency ? gw.lastLatency + " ms":"n/a");
			row.appendChild(cell);
			
			tmpRowList.push(row);
		}
		
		tmpRowList.sort(anonymox.panel.sortServiceListCompare);
		
		for ( let i = 0; i < tmpRowList.length; i++) {
			serviceList.appendChild(tmpRowList[i]);
		}
	},
	
	attachListenerToServiceList : function (gwlist) {
		let headerID = document.getElementById("anonymox-panel-gatewaylist-header-id");
		let headerLatency = document.getElementById("anonymox-panel-gatewaylist-header-latency");
		headerID.addEventListener("click", anonymox.panel.sortServiceList);
		headerLatency.addEventListener("click", anonymox.panel.sortServiceList);
	},
	
	sortServiceList : function (event) {
		let info = new anonymox.Info("panel.sortServiceList");
		
		let target = event.target;
		let targetLabel = target.getAttribute('label');
		
		if (targetLabel == anonymox.ui.servicesort.key) {
			anonymox.ui.servicesort.order *= -1;
		} else {
			anonymox.ui.servicesort.key = targetLabel;
			anonymox.ui.servicesort.order = 1;
		}
		
		
		
		try {
		
			let list = target.parentElement.parentElement;
			
			let elements = [];
			
			for ( let i = list.getRowCount() - 1; i >= 0; i--) {
				let row = list.removeItemAt(0);
				elements.push(row);
			}
			
			elements.sort(anonymox.panel.sortServiceListCompare);
			
			
			for ( let i = 0; i < elements.length; i++) {
				list.appendChild(elements[i]);
			}
		} catch(e) {
			info.debug(e);
		}
	},
	
	sortServiceListCompare : function(a, b) {
		let compare = function () { return 0; };
		if (anonymox.ui.servicesort.key == "ID") {
			compare = anonymox.panel.sortServiceListByID(a, b);
		} else if (anonymox.ui.servicesort.key == "Latency") {
			compare = anonymox.panel.sortServiceListByLatency(a, b);
		}
		return compare * anonymox.ui.servicesort.order;
	},
	
	sortServiceListByID : function(a, b) {
		let labelA = a.firstChild.getAttribute('label');
		let labelB = b.firstChild.getAttribute('label');
		return labelA.localeCompare(labelB);
	},
	
	sortServiceListByLatency : function(a, b) {
		let info = new anonymox.Info("panel.sortServiceListByLatency");
		
		let labelA = a.childNodes[4].getAttribute('label');
		let labelB = b.childNodes[4].getAttribute('label');
		if (labelA == "n/a" && labelB == "n/a") {
			return 0;
		} else if (labelB == "n/a") {
			return -1;
		} else {
			let valueA = parseFloat(labelA.split(" ")[0]);
			let valueB = parseFloat(labelB.split(" ")[0]);
			if (valueA > valueB) {
				return 1;
			}
			if (valueA < valueB) {
				return -1;
			}
			return 0;
		}
	},
	
	rebuildProfilesList : function() {
		try {
			let info = new anonymox.Info("panel.rebuildProfilesList");
			let profilesList = document.getElementById('anonymox-panel-profileslist');
			
			
			for ( let i = profilesList.getRowCount() - 1; i >= 0; i--) {
				profilesList.removeItemAt(0);
			}
			
			for (let i = 0; i < anonymox.profilemgr.list.length; i++) {
				let profile = anonymox.profilemgr.list[i];
				
				if (profile.isInternal) continue;
				
				let row = document.createElement('listitem');
				row.value = profile.site;
				
				let cell;
				
				cell = document.createElement('listcell');
				cell.setAttribute('label', profile.site!="*" ? profile.site : profile.getDisplayName());
				row.appendChild(cell);

				cell = document.createElement('listcell');
				cell.setAttribute('label', profile.gatewayFilter.enabled ? "yes" : "no");
				row.appendChild(cell);
				
				let countryCell = document.createElement('listcell');
				
				if (profile.gatewayFilter.enabled) {
					countryCell.setAttribute('label', anonymox.profilemgr.getCountryName(profile.gatewayFilter.country));
				} else {
					countryCell.setAttribute('label', "");
				}

				row.appendChild(countryCell);

				cell = document.createElement('listcell');
				cell.setAttribute('label', profile.gateway.id );
				row.appendChild(cell);
				
				profilesList.appendChild(row);
			}
		} catch(e) {
			info.error(e);
		}
	},
	
	deleteListProfile : function() {
		let profilesList = document.getElementById('anonymox-panel-profileslist');

		if (profilesList.selectedItem && profilesList.selectedItem.firstChild) {
			if (profilesList.selectedItem.value != "*") {
				anonymox.profilemgr.removeProfile(profilesList.selectedItem.value);
				profilesList.removeItemAt(profilesList.selectedIndex);
			} else {
				anonymox.ui.notifyModal("The default Profile cannot be deleted.");
			}
		}
	},

	resetGatewayStatus : function() {
		anonymox.core.resetGatewayStatus();
	},
	
	toggleLists : function() {
		let info = new anonymox.Info("toggleLists");
		
		let cont = document.getElementById('anonymox-panel-listscontainer');
		
		let isOn = !anonymox.prefs.getBoolPref("displayProfileBar");
		anonymox.prefs.setBoolPref("displayProfileBar", isOn);

		cont.hidden = !isOn;
		
		if (isOn) {
			anonymox.panel.rebuildServicesList();
			anonymox.panel.rebuildProfilesList();
		}

		info.sendMsg(anonymox.utils.msgType.usage, "features", "toggleLists");
	},

	
	
	countryListCmd : function() {
		let country = document.getElementById('anonymox-panel-pickservice-country').selectedItem.getAttribute("value");

		if (country == "more") {
			anonymox.browserapi.openURI(anonymox.config.premiumInfoURI + "morecountries", 0);
			anonymox.panel.closePanel();
		} else {
			anonymox.panel.apply();
		}
	},
	
	siteListCmd : function() {
		let siteList = document.getElementById('anonymox-panel-pickservice-site');
		
			
		if (window.anonymoxdata.profile.name != "default" && siteList.selectedItem.value == "*") {
			let promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
			if (promptService.confirm(null, "anonymoX", anonymox.localize("changeProfileDeleteHint", window.anonymoxdata.profile.name))) {
				window.anonymoxdata.profile.removeFromProfileList();
			}
			anonymox.panel.open();
			return;
		} else if (window.anonymoxdata.profile.name != siteList.selectedItem.value) {	
			anonymox.panel.apply();
			return;
		}
	},
	
	attribCmd : function(attrib, win) {
		if (anonymox.core.isPremiumAccount || attrib == "stealth") {
			anonymox.panel.apply();
		} else {
			anonymox.ui.premiumHint(window, "attrib");
			document.getElementById('anonymox-panel-attributes-fast').checked = false;
			document.getElementById('anonymox-panel-attributes-premium').checked = false;
		}
	},
	
	closePanel : function() {
		document.getElementById('anonymox-panel').hidePopup();
	},
	
	showSupportFeedback : function() {
		anonymox.browserapi.openURI(anonymox.config.supportFeedbackPageUri, 0);
		anonymox.panel.closePanel();
	},
	
	menu : function() {
		let mainMenu = document.getElementById("anonymox-mainMenu");
		mainMenu.openPopup(document.getElementById('anonymox-panel-menubutton'), 'after_start', 0, 0, false, false);
	},
	
	changeIdentity : function() {
		window.anonymoxdata.profile.rotateGateway();
	},
	
	showIdentHelp : function() {
		anonymox.browserapi.openURI(anonymox.config.identListHelpUri, 0);
		anonymox.panel.closePanel();
	},
	
	showProfilesWarning : function() {
		anonymox.browserapi.openURI(anonymox.config.profilesWarnUri, 0);
		anonymox.panel.closePanel();
	},
	
	showNetworkStatus : function() {
		anonymox.browserapi.openURI(anonymox.state.displayStatusURI, 0);
		anonymox.panel.closePanel();
	},
	
	showPreferences : function() {
		openDialog("chrome://anonymox/content/prefs.xul", "anonymox-prefs", "centerscreen, chrome, toolbar");
	},

	
	showIdentChangeCookieTip : function() {
		if (anonymox.prefs.getBoolPref("showNotifications.identChangeCookieAction"))
			anonymox.panel.setCookiePopup(true);
	},
	
	setCookieAction : function(action) {
		if (anonymoxdata.profile.cookieAction != action)
			anonymoxdata.profile.setCookieAction(action);
		setCookiePopup(false);
	},
	
	setCookiePopup : function(visibility) {
		document.getElementById("anonymox-cookieActionTip-container").hidden = !visibility;
	},
	
	cookieActionTipCloseClicked : function() {
		if (document.getElementById("anonymox-cookieActionTip-dontShowAgainCheck").checked)
			anonymox.prefs.setBoolPref("showNotifications.identChangeCookieAction", false);
		anonymox.panel.setCookiePopup(false);
	}
	
};
