/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";

var anonymox = {};

Components.utils.import("resource://anonymox/components.jsm", anonymox);

function toggleUsageCollection() {
	anonymox.prefs.setIntPref('usagecollection', document.getElementById("usageCollectionBox").checked ? 1 : 0 );
}

window.addEventListener("load", function(e) {
	
	
	updateAccountData();
	
	
	document.getElementById("usageCollectionBox").checked = anonymox.prefs.getIntPref("usagecollection", 0) == 1;
	
	updatePremiumStatus();
	
	populateNotificationSettingsList();
	
}, false);

function activatePremium() {
	anonymox.core.activatePremium(document.getElementById("activatePremium_code").value, updatePremiumStatus);
}

function updateAccountData() {
	document.getElementById("account.pass").value = anonymox.prefs.getCharPref("pass");
	document.getElementById("account.user").value = anonymox.prefs.getCharPref("user");
}

function setAccountData() {
	document.getElementById("passwordUsernameStatus").value = anonymox.localize("passwordCheckPending");
	document.getElementById("passwordUsernameStatus").style.color = "";
	anonymox.communication.csclient.Info6(
		document.getElementById("account.user").value,
		document.getElementById("account.pass").value,
		anonymox.config.version,
		anonymox.config.languageID,
		anonymox.config.osname,
		passwordUsernameCorrectSave
	).error(passwordUsernameCheckFailed);
}

function passwordUsernameCheckFailed(error) {
	let info = new anonymox.Info("prefs.passwordUsernameCheckFailed");
	if ( !error || !(error.xmessage || error.uri) ) {
		
		info.notice("Net or Server error.\n\"" + error + "\"");
		document.getElementById("passwordUsernameStatus").value = anonymox.localize("passwordCheckError");
	} else if (error.xmessage.indexOf("CE01") != -1) {
		
		document.getElementById("passwordUsernameStatus").value = anonymox.localize("passwordCheckInvalid");
	} else {
		
		document.getElementById("passwordUsernameStatus").value = error.xmessage;
	}
	document.getElementById("passwordUsernameStatus").style.color = "red";
}

function passwordUsernameCorrectSave(result) {
	anonymox.prefs.setCharPref("user", document.getElementById("account.user").value);
	anonymox.prefs.setCharPref("pass", document.getElementById("account.pass").value);
	
	document.getElementById("passwordUsernameStatus").value = "";
	anonymox.core.refreshInfo(updatePremiumStatus);
}

function updatePremiumStatus() {
	document.getElementById("activatePremiumCodeBox1").setAttribute("hidden", (true == anonymox.core.isPremiumAccount));
	document.getElementById("activatePremiumCodeBox2").setAttribute("hidden", !(true == anonymox.core.isPremiumAccount));
}

function populateNotificationSettingsList() {
	let info = new anonymox.Info("prefs.xul.populateNotificationSettingsList");
	
	let notificationPreferences = document.getElementById("notificationPreferences");
	let notificationList = document.getElementById("notificationList");
	
	for ( let i = 0, pref; pref = notificationPreferences.children[i]; i++ ) {
		let checkBox = document.createElement("checkbox");
		checkBox.setAttribute("preference", pref.id);
		checkBox.setAttribute("label", anonymox.utils.localize(pref.id + ".pref"));
		checkBox.setAttribute("checked", anonymox.prefs.getBoolPref("showNotifications." + pref.id));
		notificationList.appendChild(checkBox);
	}
}

function usageCollectionInfo() {
	anonymox.browserapi.openURI(anonymox.config.usageCollectionInfo, 0);
}
