/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";


var browserapi = {
	
	init : function(){
	
		
		if (!prefs.getBoolPref("iconCreated")){
			let win = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser");
			win.anonymox.overlay.installButton("nav-bar", "anonymox-button");
			win.anonymox.overlay.installButton("addon-bar", "anonymox-button");
			prefs.setBoolPref("iconCreated",true);
		}
	},
	
	initLate : function(){
		browserapi.shutDownObserver.register();
		browserapi.uninstallListener.register();
	},
	
	
	
	messageWindows : Array(),
	
	
	
	
	
	
	
	
	openURI : function(uri, level)
	{
		
		
		
		var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		var browserEnumerator = wm.getEnumerator("navigator:browser");
		
		var found = false;
		while (!found && browserEnumerator.hasMoreElements()) {
			var browserWin = browserEnumerator.getNext();
			var tabbrowser = browserWin.gBrowser;

			
			var numTabs = tabbrowser.browsers.length;
			for ( var index = 0; index < numTabs; index++) {
				var currentBrowser = tabbrowser.getBrowserAtIndex(index);
				if (uri == currentBrowser.currentURI.spec) {
					
					tabbrowser.selectedTab = tabbrowser.tabContainer.childNodes[index];

					
					browserWin.focus();

					found = true;
					break;
				}
			}
		}
		
		

		if (!found)
		{
			
			if (level == 1){
			
				if (Array_inArray(browserapi.messageWindows,uri)){
					browserapi.messageWindows[uri].close();
				}else
					browserapi.messageWindows[uri] = wm.getMostRecentWindow("navigator:browser")
					.open(uri, "anonymoX Message", "width=990,height=700,resizable=yes,scrollbars=yes,status=yes,location=yes")
					.focus();
		
			}else{
		
				var recentWindow = wm.getMostRecentWindow("navigator:browser");
				if (recentWindow) {
					
					recentWindow.delayedOpenTab(uri, null, null, null, null);
				} else {
					
					win.open(uri);
				}
			}
		}
	},

	closeOpenedURIs : function()	
	{
		for ( var int = 0; int < browserapi.messageWindows.length; int++) {
			var win = browserapi.messageWindows[int];
			try{
				win.close();
			}catch(e){}
		}
	},
		
	
	
	isReferrerEnabled : function(){
		return config.allPrefs.getIntPref("network.http.sendRefererHeader") != 0;
	},
	
	setReferrer : function(enabled){
		config.allPrefs.setIntPref("network.http.sendRefererHeader", (enabled ? 2 : 0));
	},
	
	
	
	setUserAgent : function(value) {
		var info = new Info("setUserAgent");
		if (value != undefined)	
			config.allPrefs.setCharPref("general.useragent.override", value);
		else
			config.allPrefs.clearUserPref("general.useragent.override");
		info.sendMsg(utils.msgType.usage, "features", "setUserAgent");
	},
	
	
	
	acceptCookies : function(accept, src){
		config.allPrefs.setIntPref("network.cookie.cookieBehavior", accept);
		info.sendMsg(utils.msgType.usage, "features", "acceptCookies " + val + "|" + src);
	},
	
	clearCookies : function(domain, src){
		var info = new Info("browserapi.clearCookies");
		info.debug(domain);
		var cookieMgr = Cc["@mozilla.org/cookiemanager;1"].getService(Ci.nsICookieManager);
		if (!domain)
			cookieMgr.removeAll();
		else
			for (var e = cookieMgr.enumerator; e.hasMoreElements();) {
				var cookie = e.getNext().QueryInterface(Components.interfaces.nsICookie);
				if (cookie.host.indexOf(domain) != -1){
					info.debug("delete " + cookie.host + ";" + cookie.name + "=" + cookie.value);
					cookieMgr.remove(cookie.host, cookie.name, cookie.path, false);
				}
			}
		info.sendMsg(utils.msgType.usage, "features", "clearCookies" + (domain?"_all":"") + "|" + src);
	},

	clearCookiesIdentChange : function (profile){
		var info = new Info("browserapi.clearCookiesIdentChange");
		
		
		var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		var win = wm.getMostRecentWindow("navigator:browser");
		if (!win) {
			new Info("ui.showTip").notice("No window found!");
			return;
		}
		
		ui.panelWindow.anonymox.panel.showIdentChangeCookieTip();
		
		info.verbose(profile.name + " (site=" + profile.site + "), " + profile.cookieAction);
		switch(profile.cookieAction){
			case 1:
				var domain = utils.getDomainPartsFromURI(win.anonymoxdata.uri);
				browserapi.clearCookies(domain[0], "identChange");
				break;
			case 2:
				browserapi.clearCookies(undefined, "identChange");
				break;
		}
	},
	
	
	
	checkForEnabledProxy : function(callback){
		
		let proxyProtocolService = Cc["@mozilla.org/network/protocol-proxy-service;1"].getService(Ci.nsIProtocolProxyService);
		let ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
		
		proxyProtocolService.asyncResolve(ioService.newURI("http://www.anonymox.net/", null, null), 0, {
			onProxyAvailable : function(aRequest, aURI, aProxyInfo, aStatus){
				callback (aProxyInfo && aProxyInfo.type != "direct");
			}
		});
	},
	
	startProxy : function(){
		config.allPrefs.setBoolPref("network.http.proxy.pipelining", true);
		browserapi.proxyService.register();
		browserapi.httpRequestObserver.register();
	},
	
	stopProxy : function(){
		config.allPrefs.setBoolPref("network.http.proxy.pipelining", false); 
		browserapi.proxyService.unregister();
		browserapi.httpRequestObserver.unregister();
	},
	
	
	proxyService : {
		proxyProtocolService : Cc["@mozilla.org/network/protocol-proxy-service;1"].getService(Ci.nsIProtocolProxyService),
		
		applyFilter : function(ps, uri, preassigedProxy) {
			var info = new Info("proxyService.applyFilter");
			try {
				if (state.isReady) {
					var profile = profilemgr.getProfileForURI(uri.spec);
					profile.isUsedNow();
					if (profile.proxyServiceInfo) {
						var pi = profile.proxyServiceInfo;
						info.verbose(uri.spec + " -> " + profile.name + " (" + pi.ip + ":" + pi.port + ")");
						return browserapi.proxyService.proxyProtocolService.newProxyInfo("http", pi.ip, pi.port, 0, null, null);
					} else {
						info.verbose(uri.spec + " -> " + profile.name + " (profile is not proxied -> DEFAULT) ");
						return preassigedProxy;
					}
				} else {
					info.debug(uri.spec + " -> RC: state.isReady==false");
					return preassigedProxy;
				}
			} catch (e) {
				info.error(e);
			}
		},

		register : function() {
			try {
				Cc["@mozilla.org/network/protocol-proxy-service;1"].getService(Ci.nsIProtocolProxyService).registerFilter(this, 0);
			} catch (e) {
				new Info("proxyService.register").error(e);
			}
		},

		unregister : function() {
			try {
				Cc["@mozilla.org/network/protocol-proxy-service;1"].getService(Ci.nsIProtocolProxyService).unregisterFilter(this);
			} catch (e) {
				new Info("proxyService.unregister").error(e);
			}
		}
	},

	
	httpRequestObserver : {
		info : new Info("httpRequestObserver"),
		
		observe : function(subject, topic, data) {
			try {
				var httpChannel = subject.QueryInterface(Ci.nsIHttpChannel);
				if (state.isReady) {
					var profile = profilemgr.getProfileForURI(httpChannel.URI.spec);
					if (profile.proxyServiceInfo) {
						
						httpChannel.setRequestHeader("Proxy-Authorization", "uid:" + config.user, false);
						if (profile.setAcceptLang && profile.gateway)
							httpChannel.setRequestHeader("Accept-Language", profile.gateway.country, false);
					}
				} else
					this.info.notice(httpChannel.URI.spec + " -> RC: state.isReady==false, this should not happen");
				
				
				if (httpChannel.URI.spec.match('^https?://([^/]+\.)?anonymox.net/')){
					httpChannel.setRequestHeader("X-AnonymoX-Auth", "uid:" + config.user, false);
					httpChannel.setRequestHeader("X-AnonymoX-Capabilities", config.capabilities, false);
				}
			} catch (e) {
				this.info.error(e);
			}
		},
		register : function() {
			try {
				var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
				observerService.addObserver(this, "http-on-modify-request", false);
			} catch (e) {
				this.info.error(e);
			}
		},
		unregister : function() {
			try {
				var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
				observerService.removeObserver(this, "http-on-modify-request");
			} catch (e) {
				
			}
		}
	},

	shutDownObserver : {
		observe : function(subject, topic, data) {
			core.shutdownStartedHook();
		},
		register : function() {
			var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
			observerService.addObserver(this, "quit-application", false);
		},
		unregister : function() {
			var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
			observerService.removeObserver(this, "quit-application");
		}
	},

	uninstallListener : {
		register : function(){
			Cu.import("resource://gre/modules/AddonManager.jsm");
			AddonManager.addAddonListener(browserapi.uninstallListener);
		},
		onUninstalling: function(addon) {
			if (addon.id == config.extensionID)
				core.uninstallHook();
		},
		onDisabling: function(addon){
			if (addon.id == config.extensionID)
				core.uninstallHook();
		}
	}
};
