/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";


var communication = {

	init : function(){
		let jQuery = utils.jQuery;
		
		let oldInfo = Info;	
		
		let subscriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
		subscriptLoader.loadSubScript("resource://anonymox/communication/thrift.js");
		subscriptLoader.loadSubScript("resource://anonymox/communication/ClientService.js");
		subscriptLoader.loadSubScript("resource://anonymox/communication/client_types.js");
		
		Info = oldInfo;
		
		let transport = new Thrift.Transport(config.masterURI);
		let protocol  = new Thrift.Protocol(transport);
		communication.csclient = new ClientServiceClient(protocol);
	},
	
	checkFailedTypes : {
		timeout : "timeout",
		wrongContent : "wrongContent",
		wrongResponseCode : "wrongResponseCode",
		networkError : "networkError",
		genericError : "genericError"
	},
	
	registerTmpFilter : function(site, gw){
		profileToUpdate = new Profile;
		profileToUpdate.name = site + "_tmp";
		profileToUpdate.site = site;
		profileToUpdate.addToProfileList();
		profileToUpdate.changedGatewayFilter(gatewayFilter);
	},
	
	httpHandlerSerialNo : 0,
	
	HttpHandler : function(){

		
		
		
		this.doRequest = function(uri, gateway, successfulfunc, failedfunc, expectedcontentregexp, maxTries){
			this.id = communication.httpHandlerSerialNo++;
			this.info = new Info("HttpHandler.doRequest " + communication.httpHandlerSerialNo);
			
			this.info.verbose(uri + " " + (gateway ? gateway.id : "") + " " + utils.displayFuncOrUndefined(successfulfunc) + " " + utils.displayFuncOrUndefined(failedfunc) + " " + expectedcontentregexp);
			
			if (maxTries == undefined) maxTries = 3;
			this.failedTries = 0;
			
			this.internalDoRequest = function() {
				try {
					var req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
					req.httpHandlerRef = this;

					if (gateway != null) {
						
						
						
						this.tmpProf = new Profile;
						this.tmpProf.isInternal = true;
						this.tmpProf.name = "httpHandler_" + this.id;
						this.tmpProf.isTemporary = true;
						this.tmpProf.uriFilter = "^" + utils.escapeForRegex(uri) + "$";
						if (gateway.id)
							this.tmpProf.gatewayFilter.id = gateway.id;
						
						var assignResult = this.tmpProf.assignGateway(false);
						
						this.tmpProf.registerProfile();
					}
					
					this.cleanUp = function() {
						if (this.tmpProf != null) {
							this.tmpProf.removeFromProfileList(true);
							this.tmpProf = null;
						}
					}
					
					this.timeout = setTimeout(function() {
						req.httpHandlerRef.info.debug("failed(timeout)");
						req.abort();
						req.httpHandlerRef.internalFailedfunc(communication.checkFailedTypes.timeout);
						req.httpHandlerRef.cleanUp();
					}, config.httpTimeoutSec * 1000 + (gateway.type != 1? 10 * 1000 : 0));
					
					req.open("GET", uri);
					
					req.onload = function() {
						this.httpHandlerRef.timeout.cancel();
						this.httpHandlerRef.cleanUp();
						if (this.status == 200) {
							var match = this.responseText.match(expectedcontentregexp);
							if (match) {
								this.httpHandlerRef.info.verbose("successful");
								successfulfunc(match);
							} else {
								this.httpHandlerRef.info.debug("failed(wrong content: Expected Regexp \"" +  expectedcontentregexp + "\" but got \"" + this.responseText + "\")");
								this.httpHandlerRef.internalFailedfunc(communication.checkFailedTypes.wrongContent, this.responseText);
							}
						} else {
							this.httpHandlerRef.info.debug("failed(wrong response code " + this.status + ")");
							this.httpHandlerRef.internalFailedfunc(communication.checkFailedTypes.wrongResponseCode, this.status);
						}
					};
					
					req.error = function() {
						this.httpHandlerRef.timeout.cancel();
						this.httpHandlerRef.cleanUp();
						this.httpHandlerRef.info.debug("failed(error " + communication.checkFailedTypes.networkError + ")");
						this.httpHandlerRef.internalFailedfunc(communication.checkFailedTypes.networkError);
					};
					
					req.send();
					
				} catch (ex) {
					this.info.notice("failed(generic error): " + ex + " code " + ex.fileName + " line " + ex.lineNumber);
					this.internalFailedfunc(communication.checkFailedTypes.genericError);
				}
			};
			
			this.internalFailedfunc = function(failedType, additionalinfo) {
				if (++this.failedTries >= maxTries) {
					failedfunc(failedType, additionalinfo);
					return;
				} else {
					
					this.newTryTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
					
					var callback = {
						notify : function() {
							this.handler.newTry();
						}
					};
					callback.handler = this;
					this.newTryTimer.initWithCallback(callback, 500, Ci.nsITimer.TYPE_ONE_SHOT);
				}
			};
			
			this.newTry = function() {
				this.internalDoRequest(uri, gateway, successfulfunc, this.internalFailedfunc, expectedcontentregexp);
			};
			
			this.newTry();
		};
	}
};
