/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";


var config = {
	init : function(){
		config.version = "2.1.1";
		config.capabilities = "oneclickactivate";
		
		
		
		config.masterURI = "https://master.anonymox.net/firefox";


		
		
		config.networkInfoURI = "http://nwi.anonymox.net/";
		config.homepageBaseUri = "http://www.anonymox.net/" + localize("lang_id") + "/";
		config.supportFeedbackPageUri = config.homepageBaseUri + "support-feedback";
		config.helpErrorPageUri = config.homepageBaseUri + "help-error";
		config.profilesWarnUri = config.homepageBaseUri + "profiles-warning";
		config.welcomeSiteUri = config.homepageBaseUri + "gettingstarted";
		config.selfCheckFailedUri = config.homepageBaseUri + "self-check-failed";
		config.identListHelpUri = config.homepageBaseUri + "identlist-help";
		config.premiumInfoURI = config.homepageBaseUri + "premiuminfo?src=";
		config.disableAdsURI = config.homepageBaseUri + "disable-ads";
		config.usageCollectionInfo = config.homepageBaseUri + "usagecollectioninfo";
		
		config.extensionID = "client@anonymox.net";
		
		config.languageID = localize("lang_id");
		
		config.installPath = undefined;		

		config.attributes = ["stealth", "fast", "premium"];
		config.premiumAttributes = ["fast", "premium"];
		
		config.osname = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULRuntime).OS;
		
		config.httpTimeoutSec = 15;
		config.refreshInfoIntervalMin = 15;
		config.netRetryWaitTimeSec = 30;
		config.usageCollectionIntervalHrs = 24 * 3;
		config.selfCheckIntervalMin = 60 * 4;
		config.selfCheckNumForAvg = 2;
		config.selfCheckTimeBetweeenChecksSec = 20;
		config.externalInfoAutoReloadMin = 15;
		config.siteIsConsideredBusyIntervalMS = 60 * 1000;
		config.changeGWThresholdScore = 1;
		
		Cu.import("resource://gre/modules/AddonManager.jsm");
		AddonManager.getAddonByID(config.extensionID, function(addon) {
			config.installPath =  addon.getResourceURI("").QueryInterface(Ci.nsIFileURL).file.path;
		});

		config.prefListener = new config.PrefListener("extensions.anonymox.",
			function(branch, name) {
				try{
					switch (name) {
						case "user":
							config.user = config.prefs.getCharPref("user");
							break;
						case "pass":
							config.pass = config.prefs.getCharPref("pass");
							break;
						case "displayIP": case "displayFlag": case "displayInStatusBar":
							if(state.isReady){
								core.getAllExternalInfoIfNeeded();
								ui.updateUIElements("extInfo");
								ui.updateUIElements("statusBarEntry");
							}
							break;
						case "debugout":
							config.debugout = config.prefs.getBoolPref("debugout");
					}
				}catch(e){
					new Info("config.prefListener").error(e);
				}
			}
		);
		config.prefListener.register();
	},
	
	PrefListener : function (branchName, func) {
		var prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
		var branch = prefService.getBranch(branchName);
		branch.QueryInterface(Ci.nsIPrefBranch2);

		this.register = function() {
			branch.addObserver("", this, false);
			branch.getChildList("", { }).forEach(function (name) { func(branch, name); });
		};

		this.unregister = function unregister() {
			if (branch)
				branch.removeObserver("", this);
		};

		this.observe = function(subject, topic, data) {
			if (topic == "nsPref:changed")
				func(branch, data);
		};
	},
	
	resetAllPrefs : function(includeAnonID) {
		config.resetPref("displayInStatusBar");
		config.resetPref("displayIP");
		config.resetPref("displayFlag");
		config.resetPref("addonBar");
		config.resetPref("debugout");
		config.resetPref("profiles.list");
		config.resetPref("usagecollection");
		config.resetPref("iconCreated");
		config.resetPref("displayProfileBar");
		config.resetPref("lastPremium");
		config.resetPref("usagecollection");
		
		config.resetPref("showNotifications.identChangeCookieAction");
		config.resetPref("showNotifications.autoGatewayChanged");
		config.resetPref("showNotifications.panelNotForThisSite");
		config.resetPref("showNotifications.configuredProxyDetectedOverride");
		config.resetPref("showNotifications.netErrorWilltryAgain");
		config.resetPref("showNotifications.newIdentityRotateReset");
		config.resetPref("showNotifications.newProfileCreated");
		
		if (includeAnonID) {
			config.resetPref("firstrun");
			config.resetPref("user");
			config.resetPref("pass");
		}
	},
	
	saveOriginalGlobalPrefsIfNotDone : function(){
		if(!prefs.prefHasUserValue("original.network.http.sendRefererHeader")){	
																				
			if(config.allPrefs.prefHasUserValue("general.useragent.override"))
				prefs.setCharPref("original.general.useragent.override", config.allPrefs.getCharPref("general.useragent.override"));
			else
				config.resetPref("original.general.useragent.override");
			prefs.setIntPref("original.network.http.sendRefererHeader", config.allPrefs.getIntPref("network.http.sendRefererHeader"));
			prefs.setBoolPref("original.network.http.proxy.pipelining", config.allPrefs.getBoolPref("network.http.proxy.pipelining"));
			prefs.setIntPref("original.network.cookie.cookieBehavior", config.allPrefs.getIntPref("network.cookie.cookieBehavior"));
		}
	},
	
	restoreGlobalPrefs : function(){
		if(prefs.prefHasUserValue("original.general.useragent.override"))
			config.allPrefs.setCharPref("general.useragent.override", prefs.getCharPref("original.general.useragent.override"));
		else
			config.resetPrefGlobal("general.useragent.override");
		config.allPrefs.setIntPref("network.http.sendRefererHeader", prefs.getIntPref("original.network.http.sendRefererHeader"));
		config.allPrefs.setBoolPref("network.http.proxy.pipelining", prefs.getBoolPref("original.network.http.proxy.pipelining"));
		config.allPrefs.setIntPref("network.cookie.cookieBehavior", prefs.getIntPref("original.network.cookie.cookieBehavior"));
		
		config.resetPref("original.network.http.sendRefererHeader");
		config.resetPref("original.general.useragent.override");
		config.resetPref("original.network.http.proxy.pipelining");
		config.resetPref("original.network.cookie.cookieBehavior");
	},
	
	resetPref: function(name){
		config.resetPrefGlobal("extensions.anonymox." + name);
	},
	
	resetPrefGlobal: function(name){
		let info = new Info("config.resetPref");
		try{
			let Application = Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication);
			let p = Application.prefs.get(name);
			if (p != null) p.reset();
		}catch(e){
			info.notice(name + " : " +  e);
		}
	}
};

var prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
config.prefs = prefService.getBranch("extensions.anonymox.");
config.allPrefs = prefService.getBranch("");


var prefs = config.prefs;
