/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";


var core = {
	startUp : function(win) {
		var info = new Info("core.startUp");
		try {
			state.isLoaded = true;

			config.init();
			utils.init(win);
			info.debug(utils.versionInfo);
			state.init();
			communication.init();
			profilemgr.init();
			browserapi.init();
			ui.newWindowOpened(win);
			
			core.upgradeSettingsIfRequired();
			
			core.switchOn();
			
			
			core.lateStartTMP = setTimeout(
				function(){
					browserapi.initLate();
					delete core.lateStartTMP;
				}, 2000
			);
			
			core.readyTimerTMP = setTimeout(
				function(){
					ui.probablyFullyLoaded = true;	
					delete core.readyTimerTMP;
				}, 5000
			);
			
		} catch (e) {
			info.error(e);
		}
	},
	
	switchOn : function() {
		var info = new Info("core.switchOn");
		try {
			state.setSystemState(state.systemStates.loading);
			
			if(state.wasAlreadyActive){
				info.debug("cleaning up");
				core.cleanState();
				ui.reActivateAll();
			}
			
			state.wasAlreadyActive = true;
			state.isStarting = true;

			config.saveOriginalGlobalPrefsIfNotDone();
			
			if (prefs.getBoolPref("firstrun")) {
				core.firstRun(); 
			} else {
				
				browserapi.checkForEnabledProxy( function(otherIsEnabled) {
					if (otherIsEnabled)
						ui.notification(0, "configuredProxyDetectedOverride");
				});
				
				core.refreshInfo(function() {
					
					
					try {
						var info = new Info("switchOn(refreshedInfo)");
						info.debug("");

						browserapi.startProxy();
						
						state.isReady = true;	

						core.selfcheckBenchmarkAllGateways();
						
						core.getAllExternalInfoIfNeeded();

						state.setSystemState(state.systemStates.ready);
						
						state.isStarting = false;
						
						
						
						
						
						if (prefs.getIntPref("usagecollection") == 1){
							
							setTimeout(function(){
								var pl = "";
								for ( var i = 0; i < profilemgr.list.length; i++) {
									if (pl!="")
										pl+="|";
									var profile = profilemgr.list[i];
									if (!profile.isInternal){
										pl+= profile.site + ";";
										pl+= profile.gatewayFilter.enabled + ";" + profilemgr.getCountryName(profile.gatewayFilter.country) + ";";
									}
								}
								communication.csclient.Log(utils.msgType.usage, "profiles", pl, config.user, undefined, function(){});
							}, 200);
						}
						
						
						if (core.showWelcomeSiteOnActivationTMP) {
							delete core.showWelcomeSiteOnActivationTMP;
							browserapi.openURI(config.welcomeSiteUri, 0);
						}
					} catch (e) {
						info.error(e);
					}
				});
			}
		} catch (e) {
			info.error(e);
		}
	},

	uninstallHook : function(){
		core.switchOff();
		config.restoreGlobalPrefs();
	},
	
	shutdownStartedHook : function(){
		
		state.isReady = false;
		browserapi.closeOpenedURIs();
	},	
	
	upgradeSettingsIfRequired : function(){
		var info = new Info("core.upgradeSettingsIfRequired");
		try{
			var lv = prefs.getCharPref("lastVersion");
		}catch(e){}
		
		if(!lv){
			info.debug("NOT upgrade from 0.9.3 or lower...");
			
			
			prefs.setCharPref("lastVersion", config.version);
		}else if(lv != config.version){
			info.debug("update from " + lv);
			
			prefs.setCharPref("lastVersion", config.version);
		}
		
		
		config.resetPref("profiles.overrideByDefault");
	},
	
	
	
	cleanState : function(){
		state.resetAll();
		delete core.gatewayListCache;
	},
	
	switchOff : function() {
		var info = new Info("core.switchOff");
		info.notice("");
		
		state.isStarting = false;
		state.isReady = false;
		
		state.setSystemState(state.systemStates.disabled);
		
		ui.deactivate();
		
		browserapi.stopProxy();
		
		
		
		
		if (core.refreshTimer) {
			core.refreshTimer.cancel();
			core.refreshTimer = null;
		}

	},
	
	
	
	refreshInfo : function(continueFunc, intervalInvoked) {
		try {
			var info = new Info("core.refreshInfo");
			var retryNum = 0;
			
			info.debug(utils.displayFuncOrUndefined(continueFunc) + ", " + intervalInvoked);

			if (intervalInvoked && !state.isReady) 
				return;

			let startCall;
			
			var successFunc = function(result){
				try {
					ui.removeNotificationForAllWindows("netErrorWilltryAgain");
					
					if (intervalInvoked && !state.isReady) 
						return;
							
					info.verbose("Received gateways:");
					for ( var i = 0; i < result.gateways.length; i++) {
						var gw = result.gateways[i];
						info.verbose(gw.id + " " + gw.ip + ":" + gw.port + " " + gw.country + " usage " + gw.usageIndex);
					}
					
					state.localExternalInfo = result.externalInfo;
					state.adBoxInfo = result.adBoxInfo;
					
					if (state.displayStatusURI != result.statusUri){
						state.displayStatusURI = result.statusUri;
						
						setTimeout(function(){	
							var win = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser");
							win.anonymox.panel.open("networkStatus");
						}, 2000);
					}
					
					if (core.lastShowPremiumInfoParts != result.showPremiumInfoParts){
						ui.rebuildPremiumInfoParts(result.showPremiumInfoParts);
						core.lastShowPremiumInfoParts == result.showPremiumInfoParts;
					}
					
					
					if(result.throttledReminderIntervalMin != ui.throttledReminderIntervalMin){
						ui.throttledReminderIntervalMin = result.throttledReminderIntervalMin;
						ui.updateThrottledReminderTimer();
					}
					
					core.updatePremiumStatus(result.premium, true);

					if (!core.gatewayListCache)
						core.gatewayListCache = new Array();
					
					info.verbose("gwlistcache: " + info.serializeArray(core.gatewayListCache, function(inp){return inp.id;}));
					info.verbose("result.gateways: " + info.serializeArray(result.gateways, function(inp){return inp.id;}));
					
					
					
					
					for ( var i = core.gatewayListCache.length - 1; i >= 0; i--) {
						var oldgw = core.gatewayListCache[i];

						var foundmatch = false;
						
						for ( var j = result.gateways.length - 1; j >=0; j--) {
							var newgw = result.gateways[j];

							if(newgw.id == oldgw.id){
								
								var propchanged = false;
								for (var property in newgw) {
									var prop_old = JSON.stringify(oldgw[property]);
									var prop_new = JSON.stringify(newgw[property]);
									if (prop_old != prop_new){
										info.verbose(newgw.id + " Property changed: " + property + " old " + prop_old + " new " + prop_new);
										oldgw[property] = newgw[property];
										propchanged = true;
									}
								  }

								foundmatch = true;
								break;
							}
						}
						if (!foundmatch)
							Array_remove(core.gatewayListCache, i);
					}
					
					
					for ( var j = result.gateways.length - 1; j >=0; j--) {
						var newgw = result.gateways[j];
						var foundmatch = false;
						
						for ( var i = core.gatewayListCache.length -1; i >= 0; i--) {
							var oldgw = core.gatewayListCache[i];
							if(newgw.id == oldgw.id){
								foundmatch = true;
								break;
							}
						}
						
						if (!foundmatch)
							core.gatewayListCache.push(newgw);
						
					}
					
					profilemgr.assignGWForAll();
					
					
					if (result.showURL != null) {
						browserapi.openURI(result.showURL, 1);
					}
					
					if (result.forceUpgradeVersion)
						ui.updateRequest(result.forceUpgradeVersion);
					
					core.refreshInfo_nextIntervalTO = setTimeout(function() {
						core.refreshInfo(null, true);
					}, 1000 * 60 * config.refreshInfoIntervalMin);
					
					
					if (continueFunc) continueFunc.call();
				} catch (e) {
					
					if (e.toString().indexOf("attempt to run compile-and-go script on a cleared scope") != -1){
						info.debug("catched 'attempt to run compile-and-go script on a cleared scope'");
						info.sendMsg(utils.msgType.notice, "errorhandler", "catched 'attempt to run compile-and-go script on a cleared scope'");
						core.refreshInfo(continueFunc, intervalInvoked);
						return;
					}
					
					info.error(e, "E005: Not able to process clientInfo.");
				}
			};
			
			var errorFunc = function(error){
				if( !(error && (error.xmessage || error.uri) )){
					info.notice("Net or Server error. Will try again in " + config.netRetryWaitTimeSec + " sec. Message: \"" + error + "\"");
					retryNum++;
					if (retryNum == 5) ui.notification(0, "netErrorWilltryAgain");
					core.refreshInfo_retryTO = setTimeout(startCall, config.netRetryWaitTimeSec * 1000);
				}else{
					if (error.uri)
						browserapi.openURI(error.uri, 1);
					else
						ui.notifyModal(localize("systemFatalError", error.xmessage));
					core.switchOff();
				}
			};

			startCall = function(){
				communication.csclient.Info6(config.user, config.pass, config.version, config.languageID, config.osname , successFunc).error(errorFunc);
			};
			
			startCall();

		} catch (e) {
			
			if (e.toString().indexOf("attempt to run compile-and-go script on a cleared scope") != -1){
				info.sendMsg(utils.msgType.notice, "errorhandler", "catched 'attempt to run compile-and-go script on a cleared scope'");
				core.refreshInfo(continueFunc, intervalInvoked);
				return;
			}
			
			if (e.toString().indexOf("second is undefined") != -1){
				info.sendMsg(utils.msgType.notice, "errorhandler", "catched 'second is undefined'");
				core.refreshInfo(continueFunc, intervalInvoked);
				return;
			}
			
			
			if (!intervalInvoked || (intervalInvoked && e.lineNumber != 210))		
				info.error(e, "E006: Not able to get clientInfo.");
		}
	},

	updatePremiumStatus : function(active, infoIsFresh){
		var info = new Info("updatePremiumStatus");
		info.verbose(active);
		
		if (core.isPremiumAccount != active){
			core.isPremiumAccount = active;

			ui.updateUIElements("icons");

			if ((core.isPremiumAccount == true) != prefs.getBoolPref("lastPremium")){
				prefs.setBoolPref("lastPremium", core.isPremiumAccount);
				if (!infoIsFresh)
					core.refreshInfo(
						function(){
							core.premiumStatusChanged();
						}
					);
				else
					core.premiumStatusChanged();
			}
		}
	},
	
	premiumStatusChanged : function(){
		var info = new Info("premiumStatusChanged");
		
		info.notice("premium account is now " + (core.isPremiumAccount?"enabled":"disabled"));
		ui.notification(1, "prem" + (core.isPremiumAccount?"_enabled":"_disabled"));
		
		if (core.isPremiumAccount){		
			
			ui.hideAdBoxIfOpen();
			ui.rebuildPremiumInfoParts(undefined);
		}else{		

			
			for ( var i = 0; i < profilemgr.list.length; i++) {
				var pro = profilemgr.list[i];

				config.premiumAttributes.forEach(function(premiumAttrib){
					pro.gatewayFilter.attributes[premiumAttrib] = undefined;
				});
			}
		}
	},

	
	firstRun : function() {
		var info = new Info("core.firstRun");
		info.debug("");
		
		var retryNum = 0;

		try {
			let startCall;
			
			var successFunc = function(result){
				config.user = result.user;
				config.pass = result.passwordPlain;
				prefs.setCharPref("user", result.user);
				prefs.setCharPref("pass", result.passwordPlain);
				
				prefs.setBoolPref("firstrun", false);
				ui.removeNotificationForAllWindows("netErrorWilltryAgain");
				
				core.showWelcomeSiteOnActivationTMP = true;
				core.switchOn();
			};

			var errorFunc = function(error){
				if( !(error && (error.xmessage || error.uri) )){
					info.notice("Net or Server error. Will try again in " + config.netRetryWaitTimeSec + " sec. Message: \"" + error + "\"");
					retryNum++;
					if (retryNum == 5) ui.notification(0, "netErrorWilltryAgain");
					core.refreshInfo_retryTO = setTimeout(startCall, config.netRetryWaitTimeSec * 1000);
				}else{
					if (error.uri)
						browserapi.openURI(error.uri, 1);
					else
						ui.notifyModal(localize("systemFatalError", error.xmessage));
					core.switchOff();
				}
			};
			
			startCall = function(){
				communication.csclient.GetAccount2(2, successFunc).error(errorFunc);
			};
			
			startCall();
			
		} catch (e) {
			info.error(e);
		}
	},

	websiteCommEventHandler : function(evt) {
		var info = new Info("websiteCommEventHandler");
	   
		var cmd = evt.target.getAttribute("cmd");
		var param1 = evt.target.getAttribute("param1");
		info.debug(cmd + " " + param1);
		
		switch(cmd){
			case "activatePremium":
				core.activatePremium(param1);
				break;
		}
	},
	
	
	activatePremium : function(code,  continueFunc){
		var info = new Info("activatePremium");

		var successFunc = function(result){
			core.updatePremiumStatus(true);
			info.debug("activated " + code);
			
			if (!state.isReady)
				core.switchOn();
			
			if (continueFunc)
				continueFunc();
		};

		var errorFunc = function(error){
			if( !(error && (error.xmessage || error.uri) )){
				info.debug(error);
				ui.notifyModal(localize("couldNotActivatePremium", "Please try again in a few minutes. Network or server error."));
			}else{
				if (error.uri)
					browserapi.openURI(error.uri, 1);
				else
					ui.notifyModal(localize("couldNotActivatePremium", error.xmessage));
			}
			
			if (continueFunc)
				continueFunc();
		};
		
		communication.csclient.ActivatePremium(code, config.user, config.pass, config.languageID, successFunc).error(errorFunc);
	},
	
	getBrowserFromChannel : function(aChannel) {
		var notificationCallbacks = aChannel.notificationCallbacks ? aChannel.notificationCallbacks : aChannel.loadGroup.notificationCallbacks;

		if (!notificationCallbacks)
			return null;

		var domWin = notificationCallbacks.getInterface(Ci.nsIDOMWindow);
		return domWin.gBrowser.getBrowserForDocument(domWin.top.document);

	},
	
	
	
	
	setGWstatus : function(gw, newStatus){
		var info = new Info("core.setGWstatus");
		
		if (newStatus){	
			if (!gw.isOnline){
				var formerStatus = gw.isOnline;
				info.verbose(gw.id + " is now online");
				if (gw == profilemgr.nullGateway)
					info.debug("Assuming global online mode from now");
				gw.isOnline = true;
				if (formerStatus == false)
					profilemgr.assignGWForAll();
			}
		}else{	
			setGWstatus_helper_setOff = function(gw){
				if (gw.isOnline == undefined || gw.isOnline == true){
					info.debug(gw.id + " failure. Temporary disabling this gateway(until next selfCheck).");
					gw.isOnline = false;
					profilemgr.assignGWForAll();	
				}
			};
			
			if (gw == profilemgr.nullGateway){
				if (gw.isOnline == undefined || gw.isOnline == true)
					info.notice("Assuming global offline mode from now");
				setGWstatus_helper_setOff(gw);	
			}else{
				if (profilemgr.nullGateway.isOnline){
					setGWstatus_helper_setOff(gw);	
					info.sendMsg(utils.msgType.notice, "selfcheckfailed", gw.id);
				}else
					info.debug("not disabling failed " + gw.id + " because we are in offline mode");
			}
		}
	},
	
	selfcheckBenchmarkAllGateways_callbackFactory : function(gateway){
		return function(){
			try{
				if(!state.isReady) return;
				if (Array_inArray(core.gatewayListCache, gateway) || gateway == profilemgr.nullGateway)
					core.selfCheckGW(gateway, true);
				else
					new Info("selfcheckBenchmarkAllGateways_callback").debug("Gateway " + gateway.id + " does not exist anymore, skipping test");
			}catch(e){
				info.error(e);
			}
		};
	},
	
	selfcheckBenchmarkAllGateways : function() {
		if (config.disableSelfCheck) return;
		var info = new Info("core.selfcheckBenchmarkAllGateways");
		try {
			var offset = 0;
			var nextInterval;
			
			let scheduleCheck = function (gateway) {
				gateway.TMP_nextCheckTimer = new Array();
				offset++;
				for ( var i = 0; i < config.selfCheckNumForAvg; i++) {
					nextInterval = 1000 * offset + i * 1000 * config.selfCheckTimeBetweeenChecksSec;
					
					gateway.TMP_nextCheckTimer[i] = setTimeout(
						core.selfcheckBenchmarkAllGateways_callbackFactory(gateway),
						nextInterval
					);
					info.verbose("check for " + gateway.id + " scheduled in " + nextInterval + " ms");
				}
			};
			
			scheduleCheck(profilemgr.nullGateway);
			
			core.gatewayListCache.forEach(
				function(gw){
					scheduleCheck(gw);
				}
			);
			
			var reassignTime = nextInterval + 10 * 1000;
			
			core.reassignTimeout = setTimeout(function(){
				if (state.isReady){
					info.debug("Reassigning after selfcheck");
					profilemgr.assignGWForAll();
				}
			}, reassignTime);
			
			info.verbose("Gateway reasignment scheduled in " + reassignTime + " ms");
			
			core.nextselfcheckBenchmarkAllGatewaysTimer = setTimeout(
				function(){
					core.selfcheckBenchmarkAllGateways();
				}, 1000 * 60 * config.selfCheckIntervalMin
			);
		}catch(e){
			info.error(e);
		}
	},
	
	selfCheckGW : function(gateway, useForMeasure){
		if (config.disableSelfCheck) return;
		var info = new Info("core.selfCheckGW " + gateway.id);
		info.verbose("");
		
		if(!state.isReady) return;
		try{
			if (!gateway.isCurSelfChecked)
			{
				gateway.isCurSelfChecked = true;
				var hh = new communication.HttpHandler();
				var startTime = new Date().getTime();
				
				var successFunc = function(text){
					if(!state.isReady) return;
					
					gateway.isCurSelfChecked = false;
					
					var lat = new Date().getTime() - startTime;
					
					info.verbose("success, " + lat + " ms");
					
					core.setGWstatus(gateway, true);
					
					if (useForMeasure){
						if (!gateway.lastLatencyArray)
							gateway.lastLatencyArray = new Array();
						
						gateway.lastLatencyArray.push(lat);
						
						if (gateway.lastLatencyArray.length == config.selfCheckNumForAvg){
							var sum=0;
							for ( var i = 0; i < gateway.lastLatencyArray.length; i++) {
								var el = gateway.lastLatencyArray[i];
								sum += el;
							}
							gateway.lastLatency = sum / config.selfCheckNumForAvg;
							info.verbose("Latency Avg for " + gateway.id  + " : " + gateway.lastLatency);
						}
					}
				};
				
				var failedFunc = function(a,b){	
					if(!state.isReady) return;
					info.verbose("failed");
					gateway.isCurSelfChecked = false;
					core.setGWstatus(gateway, false);
				};

				var filter = (gateway != profilemgr.nullGateway ? /^OK$/ : /.*/);
				
				
				
				hh.doRequest(config.networkInfoURI + "selfcheck?gw=" + gateway.id, gateway, successFunc, failedFunc, filter);
			}else{
				info.verbose("Already being self-checked. skip");
			}
		}catch(e){
			info.error(e);
		}
	},
	
	resetGatewayStatus : function(){
		var info = new Info("core.resetGatewayStatus");
		try{
			core.gatewayListCache.forEach(
				function(gw){
					delete gw.isOnline;
				}
			);
			profilemgr.assignGWForAll();
			core.selfcheckBenchmarkAllGateways();
		}catch(e){
			info.error(e);
		}
	},
	
	
	
	
	getAllExternalInfoIfNeeded : function(autorepeat){
		var info = new Info("core.getAllExternalInfoIfNeeded");
		info.verbose("");
		if (prefs.getBoolPref("displayIP") || prefs.getBoolPref("displayFlag")){
			for ( var i = 0; i < profilemgr.list.length; i++) {
				var profile = profilemgr.list[i];
				core.getExternalInfo(profile.gateway);
			}
			ui.updateUIElements("extInfo");
		}
		
		if (autorepeat){
			core.getAllExternalInfoIfNeededAutoTO = setTimeout(function(){
				core.getAllExternalInfoIfNeeded(true);
			}, config.externalInfoAutoReloadMin * 60 * 1000);
		}
	},
	
	getExternalInfoIfNeeded : function(gateway){
		var info = new Info("getExternalInfoIfNeeded");
		if (prefs.getBoolPref("displayIP") || prefs.getBoolPref("displayFlag")){
			info.verbose("");
			core.getExternalInfo(gateway);
		}else
			info.verbose("exit, not needed");
	},
	
	getExternalInfo : function(gateway, ani) {
		var info = new Info("getExternalInfo");
		info.verbose(gateway.id);

		if (!gateway.externalInfo) gateway.externalInfo = {};
		
		if (!state.isReady){	
			info.debug("core is not ready -> abort");
			return;
		}
		
		if (gateway.externalInfo.status == 1) {
			info.verbose("already being processed -> abort");
			ui.externalInfoUpdated(gateway);
			return;		
		}
		
		gateway.externalInfo.status = 1;
		ui.externalInfoUpdated(gateway);
		
		var failedFunc = function(stat, add){
			if (!state.isReady) return;
			gateway.externalInfo.status = -1;
			ui.externalInfoUpdated(gateway);
			if (!core.sentExtInfoError){
				info.sendMsg(utils.msgType.notice, "refreshExternalInfo", "failed|" + gateway.id + "|" + stat + "|" + add);
				core.sentExtInfoError=true;
			}
		};

		var successFunc = function(match) {
			if (!state.isReady) return;
			gateway.externalInfo.status = 2;
			gateway.externalInfo.ip = match[1];
			gateway.externalInfo.country = match[2];
			info.verbose("successful-> " + gateway.externalInfo.ip + ", " + gateway.externalInfo.country);
			ui.externalInfoUpdated(gateway, ani);
		};

		var hh = new communication.HttpHandler();
		hh.doRequest(config.networkInfoURI + "externalinfo?gw=" + gateway.id, gateway, successFunc, failedFunc, "^([^ ]{1,15}) (.{2})$");
	}
};
