/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";


var Profile = function() {
	

	
	
	
	this.name = undefined, 	
	
	

	
	this.site = undefined, 			
	this.uriFilterType = undefined, 	

	this.uriFilter = undefined, 		

	
	this.gatewayFilter = 	
	{
		enabled : undefined,	
		country : undefined,	
		id : undefined,			
		wantId : undefined,		
		attributes : {},		
		lastGW : undefined		
	},
	
	this.cookieAction = 0,	
	
	
	
	
	this.gateway = undefined, 	
	
	this.proxyServiceInfo = 		
	{
		ip : undefined,
		port : undefined
	},
	this.externalInfo = 	
	{
		ip : undefined,
		country : undefined		
	},
	this.gatewayRotateList = undefined;		
	this.gatewayRotateHistory = undefined;	

	this.lastUsedTime = 0;
	this.noMatchingServiceFoundMsgPostponed = false;
	this.autoGatewayChangedMsgPostponed = false;
	
	
	
	this.getDisplayName = function(){
		switch(this.name){
			case "default": case "special": case "local":
				return localize("profileName_" + this.name);
				break;
			default:
				return profilemgr.getReadableName(this.name);
		}
	};
	
	this.registerProfile = function() {
		var info = new Info("profilemgr.registerProfile");
		
		info.verbose(this.name + " isInternal " + (this.isInternal ? "1" : "0")  + " isTmp " + (this.isTemporary ? "1" : "0"));
		if (this.site != "*") {
			profilemgr.list.unshift(this);
		} else {
			
			profilemgr.list.push(this);
		}
		if (!this.isInternal && !this.isTemporary)
			profilemgr.save();
		if (!this.isTemporary && state.isReady){
			this.assignUriFilter();
			ui.profileChangedHook("add", this);
		}
	};

	this.removeFromProfileList = function(isTemporary) {
		var found = false;
		for ( var i = 0; i < profilemgr.list.length; i++) {
			var p = profilemgr.list[i];
			if (p == this) {
				found = true;
				profilemgr.list.splice(i, 1);
				if (!isTemporary){
					profilemgr.save();
					ui.profileChangedHook("remove", p);
				}
			}
		}
		
		if (!found)
			throw ("E003: No such profile found");
		
		if (!isTemporary)
			profilemgr.save();
	};

	this.isUsedNow = function(){
		let info = new Info("Profile.isUsedNow");
		
		this.lastUsedTime = new Date().getTime();
		if (this.noMatchingServiceFoundMsgPostponed){
			this.noMatchingServiceFoundMsgPostponed = false;
			if (this.autoGatewayChangedMsgPostponed){	
				this.autoGatewayChangedMsgPostponed = false;
				info.debug("Dropping postponed autoGatewayChangedMsg");
			}
			this.showNoMatchingServiceFound();
		}
		
		if (this.autoGatewayChangedMsgPostponed){
			info.debug("Displaying postponed autoGatewayChangedMsg");
			this.autoGatewayChangedMsgPostponed = false;
			this.showAutoGatewayChanged();
		}
	};
	
	this.showNoMatchingServiceFound = function(){
		if (!this.hideNoMatchingServiceFoundTakeDefault){
			let thisProfile = this;
			ui.notification(1, "noMatchingServiceFoundTakeDefault", this.name, [
                	{
						label: "Ignore for session",
						accessKey: "i",
						callback: function() {
							thisProfile.hideNoMatchingServiceFoundTakeDefault = true;
						}
					},
					{
						label: "Delete profile",
						accessKey: "d",
						callback: function() {
							thisProfile.removeFromProfileList();
						}
					}
                ]
			);
		}
	};
	
	
	
	
	this.assignGateway = function(takeDefaultIfNoMatch, invokedInternally) {
		let info = new Info("Profile.assignGateway " + this.name);
		
		
		let assignResult = this.assignGatewayInternal(this.gatewayFilter, invokedInternally, true);
		
		this.lastGatewayRotateList = this.gatewayRotateList;
		
		if (assignResult)
			return true;
		else {
			if (takeDefaultIfNoMatch){
				
				info.warning("Could not assign gateway, assigning default");
				
				if (new Date().getTime() - this.lastUsedTime < config.siteIsConsideredBusyIntervalMS)
					this.showNoMatchingServiceFound();
				else{
					info.debug("postponing noMatchingServiceFound");
					this.noMatchingServiceFoundMsgPostponed = true;
				}
				
				let assignResult = this.assignGatewayInternal(profilemgr.defaultFilter, invokedInternally, false);
				
				if ( !assignResult ){
					info.warning("Cannot select a default. Exiting with error.");
					
					if (state.isReady){
						if (profilemgr.nullGateway.isOnline){	
							browserapi.openURI(config.selfCheckFailedUri, 1);
							ui.notification(1, "errorNoProxies");
						} else
							info.error("E008: Got no working Service to activate. Exit", localize("noServicesError"));
						core.switchOff();
					}
				}
			}
			return false;
		}
	};
	
	
	this.getGatewaysForFilter = function(filter){
		let info = new Info("Profile.getGatewaysForFilter");
		
		info.verbose("Finding gateway for filter " + profilemgr.serializeFilter(filter));
		
		
		
		
		if (filter.enabled == false)
			return [profilemgr.nullGateway];
		
		
		if (filter.id){
			if (filter.id == profilemgr.nullGateway.id)
				return [profilemgr.nullGateway];
			else {
				for ( let i = 0; i < core.gatewayListCache.length; i++) {
					let gateway = core.gatewayListCache[i];
					
					if (filter.id == gateway.id)
						return [gateway];
				};
				
				
				throw "E009: Gateway id '" + filter.id + "' not found";
			}
		}
		
		
		
		let filterSelectedGWs = new Array();
		for ( let i = 0; i < core.gatewayListCache.length; i++) {
			let gateway = core.gatewayListCache[i];
			
			
			if (
				
				(gateway.isOnline || gateway.isOnline == undefined )
				&&
				
				(!filter.country || filter.country == gateway.country)
			)
			{
				
				let matchAttribFilter = true;
				config.attributes.forEach(function(attribName){
					if (filter.attributes[attribName] != undefined && filter.attributes[attribName] != gateway.attributes[attribName])
						matchAttribFilter = false;
				});
				if (matchAttribFilter)
					filterSelectedGWs.push(gateway);
				
				
			}
			
			
		}
		return filterSelectedGWs;
	};
	
	
	
	this.assignGatewayInternal = function(filter, invokedInternally, notifyUserOnGwChange) {
		let info = new Info("Profile.assignGatewayInternal " + this.name );
		
		let chooseGW = null, oldGW = this.gateway;
		
		this.gatewayRotateList = this.getGatewaysForFilter(filter);
		
		
		
		info.verbose("Filtered matches(=gatewayRotateList): " + info.serializeArray(this.gatewayRotateList, function(gw){return gw.id;}));

		if (this.gatewayRotateList.length == 1){
			chooseGW = this.gatewayRotateList[0];
		} else if (this.gatewayRotateList.length == 0){
			info.verbose("No gateways left after pre- and history-selection. Return.");
			return false;
		} else if (this.gatewayRotateList.length > 1){
			
			
			
			
			let highestCheckTime;
			for ( let i = 0; i < this.gatewayRotateList.length; i++) {
				var gw = this.gatewayRotateList[i];
				if (gw.lastLatency >  highestCheckTime || !highestCheckTime)
					highestCheckTime = gw.lastLatency;
			}
			
			let lastGW;
			
			for ( let  i = 0; i < this.gatewayRotateList.length; i++) {
				let gw = this.gatewayRotateList[i];
				
				let latencyScore = (1 - gw.lastLatency / highestCheckTime).toFixed(2);
				if (isNaN(latencyScore)) latencyScore = 0;
				
				let usageScore = (1 - gw.usageIndex / 100).toFixed(2);
				if (isNaN(usageScore)) usageScore = 0.5;

				let countryScore = gw.country == state.localExternalInfo.country ? 1:0;
				
				if (gw.id == filter.lastGW)
					lastGW = gw;
				
				gw.score = parseFloat(latencyScore) + parseFloat(usageScore) + parseFloat(countryScore);
				
				
				let randScore = (config.user % 100 < 50 ? i / 100000 : - i / 100000);
				gw.score += parseFloat(randScore);
				
				gw.score = gw.score.toFixed(6);
				
				
			}
			
			this.gatewayRotateList.sort(function(a, b){return b.score - a.score});
			
			
			
			if (filter.wantId){
				
				for ( let i = 0; i < this.gatewayRotateList.length; i++) {
					let gateway = this.gatewayRotateList[i];
					
					if (filter.wantId == gateway.id)
						chooseGW = gateway;
				}
				
				if (!chooseGW)
					info.debug("Ignoring wantId to get more results");
			}

			if (!chooseGW){
				
				
				
				
				
				
				
				let historyFiltered = new Array();
				
				
				for ( let i = 0; i < this.gatewayRotateList.length; i++) {
					var gw = this.gatewayRotateList[i];
					if (!this.gatewayRotateHistory || this.gatewayRotateHistory.indexOf(gw.id) == -1 ){
						historyFiltered.push(gw);
					}
				}
				
				let tmpDisallowGW;
				if (historyFiltered.length == 0){
					
					let lastAddedGW = this.gatewayRotateHistory[this.gatewayRotateHistory.length - 1];
					this.gatewayRotateHistory = new Array();
					info.debug("No gateways historyFilter-selected. Re-Init gatewayRotateHistory, tmpDisallow " + lastAddedGW);
					ui.notification(0, "newIdentityRotateReset");
					tmpDisallowGW = lastAddedGW;
					historyFiltered = this.gatewayRotateList;
				}
	
				info.verbose("Match after history filtering: " + info.serializeArray(historyFiltered, function(gw){return gw.id;}) + ", gatewayRotateHistory: " + info.serializeArray(this.gatewayRotateHistory));
				
				
	
				let bestGW = historyFiltered[0];
				info.verbose("lastGW " + (lastGW ? lastGW.id + " " + lastGW.score : "-") + ", bestGW " + bestGW.id + " " + bestGW.score);

				if (lastGW){
					let lastGWvalid = Array_inArray(historyFiltered, lastGW) && tmpDisallowGW != lastGW.id;
					let lastGWGoodEnough = bestGW.score < parseFloat(lastGW.score) + parseFloat(config.changeGWThresholdScore);
					if (lastGWGoodEnough && lastGWvalid )
						chooseGW = lastGW;
					else
						chooseGW = bestGW;
				} else
					chooseGW = bestGW;
			}
		}
		
		
		

		info.verbose("Assigned gateway for " + this.name + " to " + chooseGW.id + 
				" (last was " + (oldGW ? oldGW.id : "none") + ") Filter: " + profilemgr.serializeFilter(filter));
		
		if (oldGW != chooseGW && !this.isTemporary)
			info.debug("Changed gateway for " + this.name + " to " + chooseGW.id + 
					" (last was " + (oldGW ? oldGW.id : "none") + ") Filter: " + profilemgr.serializeFilter(filter));

		this.gateway = chooseGW;
		this.gatewayFilter.lastGW = chooseGW.id;
		
		if (chooseGW == profilemgr.nullGateway){
			this.proxyServiceInfo = undefined;
			this.gatewayRotateList = undefined;
		}else{
			this.proxyServiceInfo = {
				ip : this.gateway.ip,
				port : this.gateway.port
			};
		}
	
		if (!this.isTemporary){
			
			
			var gw = this.gateway;
			setTimeout(function(){
				core.selfCheckGW(gw);
			}, 10);

			
			
			
			if (oldGW != this.gateway || !Array_equal(this.lastGatewayRotateList, this.gatewayRotateList))
				if (state.isReady)
					ui.profileChangedHook("change", this);
			
			if (oldGW != this.gateway){

				
				if (state.isReady)
					core.getExternalInfoIfNeeded(this.gateway);

				
				if (!this.isInternal)
					profilemgr.save(this);
				
				
				if (oldGW){
					if (invokedInternally){
						if (notifyUserOnGwChange){	
							this.showAutoGatewayChanged = function(){
								ui.notification(0, "autoGatewayChanged", [this.name, oldGW.id]);
							};
							if (new Date().getTime() - this.lastUsedTime < config.siteIsConsideredBusyIntervalMS)
								this.showAutoGatewayChanged();
							else{
								info.debug("postponing autoGatewayChanged");
								this.autoGatewayChangedMsgPostponed = true;
							}
						}
					}else
						browserapi.clearCookiesIdentChange(this);
				}
			}
		}
		
		return true;
	};

	this.assignUriFilter = function() {
		switch (this.uriFilterType) {
			case profilemgr.uriFilterType.standard:
				this.uriFilter = RegExp("^(?:https?|ftp|ws)://(?:[^/]+\\.)?" + utils.escapeForRegex(this.site) + "(?::\\d+)?/");
				break;
		}
	};
	
	this.wantGateway = function(gw){
		var info = new Info("Profile.wantGateway");
		info.debug(gw.id);
		this.gatewayFilter.wantId = gw.id;

		this.assignGateway(true);
	};
	
	this.rotateGateway = function(){
		var info = new Info("Profile.rotateGateway");
		try{
			info.sendMsg(utils.msgType.usage, "features", "changeIdentity");
			
			if (!this.gatewayRotateHistory){
				info.debug("Reinit");
				this.gatewayRotateHistory = new Array();
			}
			info.debug("Push to gatewayRotateHistory: " + this.gateway.id);
			this.gatewayRotateHistory.push(this.gateway.id);
			this.gatewayFilter.wantId = undefined;
			
			this.assignGateway(true);
		}catch(e){
			info.error(e);
		}
	};
	
	this.setCookieAction = function(op){
		this.cookieAction = op;
		profilemgr.save(this);
	};
};
