/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";


var ui = {
	adBoxTimer : undefined,
	notifications : {},
	panelWindow : undefined,	
	throttledReminderIntervalMin : undefined,
	throttledReminderTimer : undefined,
	
	servicesort : {
		key : "ID",
		order : 1 
	},
	
	
	
	
	newWindowOpened : function(win) {
		var info = new Info("newWindowOpened");
		info.verbose("");
		try{
			
			if (!win.anonymoxdata) win.anonymoxdata = {};

			if (state.isReady)
				ui.activate(win);
			
			ui.initNotifications(win);
			ui.updateStatusLabels(win);
			ui.updateUIElements(null, win);
		}catch(e){
			info.error(e);
		}
	},
	
	reActivateAll : function(win){
		ui.safeWindowIterator(function(win){
			ui.newWindowOpened(win);
		}, true);
	},
	
	
	safeWindowIterator : function(func, forceAll){
		var browserEnumerator = Cc["@mozilla.org/appshell/window-mediator;1"].
			getService(Ci.nsIWindowMediator).getEnumerator("navigator:browser");
		while (browserEnumerator.hasMoreElements()) {
			var win = browserEnumerator.getNext();
			if (win.anonymoxdata || forceAll)	
				func(win);
		}
	},

	
	activate : function(win) {
		var info = new Info("activate");
		info.verbose("");
		if (!win) {	
			ui.safeWindowIterator(function(iwin){
				ui.activate(iwin);
			});
		} else {	
			ui.locationOrProfileChange(win);
			win.gBrowser.addProgressListener(win.anonymox.overlay.progressListener);
			let gwlist = win.document.getElementById("anonymox-panel-gatewaylist");
			win.anonymox.panel.attachListenerToServiceList(gwlist);
		}
	},
	
	initNotifications : function(win) {
		var info = new Info("initNotifications");
		
		const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
		let notificationbox = win.document.createElementNS(XULNS, "notificationbox");
		notificationbox.id = "anonymox-notifications";
		
		let bottombox = win.document.getElementById("browser");
		bottombox.parentNode.insertBefore(notificationbox, bottombox);
		
		
		notificationbox.clientTop;
		
		win.document.getElementById("anonymox-notifications").notificationsHidden = false;

		
		for (var n in ui.notifications) {
			try {
				let currentNotification = ui.notifications[n];
				if (currentNotification.global) {
					ui.addNotificationForWindow(win, currentNotification);
				}
			} catch(e) {
				info.debug(e);
			}
		}
	},
	
	deactivate : function(win){
		var info = new Info("ui.deactivate");
		if (!win){
			ui.safeWindowIterator(function(iwin){
				ui.deactivate(iwin);
			});
		}else{
			ui.updateStatusLabels(win);
			win.gBrowser.removeProgressListener(win.anonymox.overlay.progressListener);
			win.anonymoxdata = undefined;

			
			for ( var i = 0; i < win.gBrowser.browsers.length; i++) {
				var browser = win.gBrowser.browsers[i];
				browser.anonymoxdata = undefined;
			}
		}
	},
	
	
	
	externalInfoUpdated : function(gateway, ani){
		var info = new Info("ui.externalInfoUpdated");
		try{
			var update = function(){
				info.verbose(gateway.id + " " + gateway.externalInfo.status + " " + (gateway.externalInfo.status == 2 ? gateway.externalInfo.ip + "(" + gateway.externalInfo.country + ")": ""));
				
				ui.safeWindowIterator(function(iwin){
					if (iwin.anonymoxdata.profile && iwin.anonymoxdata.profile.gateway == gateway)
						ui.updateExternalInfo(iwin);
				});
			};
			
			if (ani){
				gateway.aniTime = setTimeout(function(){
					update();
				}, 300);
			}else
				update();
		} catch (e) {
			info.error(e);
		}
	},
	
	updateExternalInfo : function(win){
		var info = new Info("ui.updateExternalInfo");
		if (!win.anonymoxdata) return;
		let gateway = win.anonymoxdata.profile.gateway;
		info.verbose("gateway " + gateway.id + " " + (gateway.externalInfo?gateway.externalInfo.ip:"---"));
		
		if (prefs.getBoolPref("displayIP")) {
			var bar_iplabel = win.document.getElementById('anonymox-externalIp');
			var panel_iplabel = win.document.getElementById('anonymox-infopanel-iplabel');

			if (!gateway.externalInfo) {
				panel_iplabel.value = bar_iplabel.value = "...";
			} else {
				switch (gateway.externalInfo.status) {
					case 1:			
						if (bar_iplabel.value=="")	
							panel_iplabel.value = bar_iplabel.value = localize("IPLoading");
						panel_iplabel.style.color = "orange";
						bar_iplabel.style.color = "orange";
						break;
					case 2:			
						panel_iplabel.value = bar_iplabel.value = localize("IP") + " " + gateway.externalInfo.ip;
						panel_iplabel.style.color = "green";
						bar_iplabel.style.color = "green";
						break;
					case -1:		
						panel_iplabel.value = bar_iplabel.value = localize("IPLoadFailed");
						panel_iplabel.style.color = "red";
						bar_iplabel.style.color = "red";
						break;
				}
			}
		}
		
		if (prefs.getBoolPref("displayFlag")) {
			var bar_flag = win.document.getElementById('anonymox-externalFlag');
			var panel_flag = win.document.getElementById('anonymox-infopanel-flag');

			if (!gateway.externalInfo) {
				panel_flag.src = bar_flag.src = "";
			} else {
				switch (gateway.externalInfo.status) {
					case 1:			
						panel_flag.value = bar_flag.value = "??";
						break;
					case 2:			
						panel_flag.src = bar_flag.src = "chrome://anonymox/skin/flags/" + gateway.externalInfo.country + ".png";
						break;
					case -1:		
						panel_flag.src = bar_flag.src="";
						break;
				}
			}
		}
	},

	

	locationOrProfileChange : function(win, browser, uri, profileChange){
		try{
			var info = new Info("ui.locationOrProfileChange");
			
			if (!browser){
				browser = win.gBrowser.selectedBrowser;
				if (!browser){
					info.error({e:"ABORT: Cannot determine browser, URI " + uri});
					return;
				}
			}
		
			if (!uri){
				uri = win.gBrowser.selectedBrowser.currentURI.spec;
				if (!uri){
					info.error({e:"ABORT: Cannot determine URI for current window"});
					return;
				}
			}

			var profile = profilemgr.getProfileForURI(uri);
			
			if (!profileChange)
				profile.isUsedNow();
			
			info.verbose("Profile: " + profile.name);
			
			
			win.anonymoxdata.uri = uri;	
			var oldProfile = win.anonymoxdata.profile;

			if (oldProfile != profile){
				info.verbose("New profile for window: " + profile.name + " (uri " + uri + ")");
				win.anonymoxdata.profile = profile;
				if (ui.panelWindow && win == ui.panelWindow)
					ui.panelWindow.anonymox.panel.profileChangedHook();
			}

			ui.updateStatusLabels(win);
			ui.updateExternalInfo(win);
			
			if (state.adBoxInfo)
				ui.adBoxURIChange();
			
			
			if (!browser.anonymoxdata) browser.anonymoxdata = {};
			if (browser.anonymoxdata.profile != profile){
				info.verbose("New profile for browser: " + profile.name + " (uri " + uri + ")");
				browser.anonymoxdata.profile = profile;
			}
			
			ui.removeNonPersistantNotificationsForAllWindows();
		}catch(e){
			info.error(e);
		}
		
	},

	
	
	profileChangedHook : function(topic, profile){
		var info = new Info("ui.profileChangedHook");
		try{
			info.verbose(topic + " " + profile.name + " (gwid " + profile.gateway.id + ")");
			ui.safeWindowIterator(function(wind){
				if (topic == "change"){
					if (wind.anonymoxdata.profile == profile) {
						ui.updateStatusLabels(wind);
						wind.anonymox.panel.profileChangedHook();
					}
				}else if (topic == "add" || topic == "remove"){
					ui.locationOrProfileChange(wind, undefined, undefined, true);
				}
			});
		}catch(e){
			info.error(e);
		}
	},
	
	
	
	adBoxInfoPageChangeCount : 0,
	
	adBoxURIChange : function(){
		if (ui.adBoxInfoPageChangeCount++ >= state.adBoxInfo.pageChangesPerAd){
			ui.adBoxInfoPageChangeCount = 0;
			ui.showAdBox();
		}
		else
			ui.hideAdBoxIfOpen();
	},
	
	showAdBox : function(){
		var win = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser"),
			info = new Info("ui.showAdBox"),
			width = win.document.width,
			height = win.document.height;

		if (win.anonymoxdata.profile.gateway != profilemgr.nullGateway){
			ui.hideAdBoxIfOpen();
			ui.adBoxPanel_frame = win.document.getElementById("anonymox-adbox-frame");
			ui.adBoxPanel = win.document.getElementById("anonymox-adbox");

			if (ui.adBoxPanel_frame.getAttribute("src") != state.adBoxInfo.frameURL) {
				ui.adBoxPanel_frame.setAttribute("src", state.adBoxInfo.frameURL);
			} else {
				ui.adBoxPanel_frame.webNavigation.loadURI(state.adBoxInfo.frameURL, Ci.nsIWebNavigation.LOAD_FLAGS_NONE, null, null, null);
			}

			ui.adBoxPanel_frame.addEventListener("DOMContentLoaded",function(){
				if(ui.adBoxPanel.state=="closed"){
					ui.adBoxPanel.setAttribute("hidden", false);
					ui.adBoxPanel.openPopup(win.document.getElementById('addon-bar'), 'before_start', 200, 0, false, false);
					
					var framewidth = ui.adBoxPanel_frame.contentDocument.body.getAttribute("frame_width");
					var frameheight = ui.adBoxPanel_frame.contentDocument.body.getAttribute("frame_height");

					
					if (width < framewidth * 2 || height < frameheight * 3) {
						info.debug("window is to small to display pop-up - closing.");
						ui.hideAdBoxIfOpen();
					}

					if (!framewidth || !frameheight){	
						ui.hideAdBoxIfOpen();
					}else{
						ui.adBoxPanel_frame.setAttribute("width", framewidth ? framewidth : 500);
						ui.adBoxPanel_frame.setAttribute("height", frameheight ? frameheight : 200);
					}
				}
			});
			
		}
	},
	
	hideAdBoxIfOpen : function(){
		if (ui.adBoxPanel)
			ui.adBoxPanel.setAttribute("hidden", true);
	},
	
	disableAdClicked : function(){
		ui.adBoxPanel.hidePopup();
		browserapi.openURI(config.disableAdsURI, 0);
	},
	
	
	
	
	rebuildPremiumInfoParts : function(showPremiumInfoParts) {
		var info = new Info("rebuildPremiumInfoParts");
		info.verbose(showPremiumInfoParts);
		
		let parts = showPremiumInfoParts ? showPremiumInfoParts.split(",") : [];
		
		if (Array_inArray(parts, "morecountries")) {
			ui.showMoreCountries = true;
		} else {
			ui.showMoreCountries = false;
		}
			
	},
	
	showLimitNotification :function() {
		if(!state.isReady) return;
		
		var win = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser");
		if(ui.throttledReminderIntervalMin && win.anonymoxdata.profile.gateway != profilemgr.nullGateway){
			ui.notification(1, "speedLimit", undefined,
				{
					label: localize("speedLimitUpdateButton"),
					accessKey: "u",
					callback: function() {
						browserapi.openURI(config.premiumInfoURI + "throttledReminder", 0);
					}
				}
			);
		}
		ui.updateThrottledReminderTimer();
	},
	
	updateThrottledReminderTimer : function(){
		var info = new Info("updateThrottledReminderTimer");
		if(!ui.throttledReminderIntervalMin){
			info.debug("cancelling timer");
			if (ui.throttledReminderTimer)
				ui.throttledReminderTimer.cancel();
		}else{	
			if (!ui.throttledReminderTimer){
				info.debug("setting up timer to " + ui.throttledReminderIntervalMin + " min");
				ui.throttledReminderTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
				ui.throttledReminderTimer.initWithCallback({
					notify : function() {
						ui.showLimitNotification();
					}
				}, 1000 * 60 * ui.throttledReminderIntervalMin, Ci.nsITimer.TYPE_REPEATING_SLACK);
			}else if (ui.throttledReminderTimer.delay != 1000 * 60 * ui.throttledReminderIntervalMin){
				ui.throttledReminderTimer.delay = 1000 * 60 * ui.throttledReminderIntervalMin;
				info.debug("timer interval updated to " + ui.throttledReminderIntervalMin + " min");
			}
		}
	},
	
	
	
	systemStateUpdateHook : function (){
		var info = new Info("ui.systemStateUpdated");
		
		try{
			info.debug(state.systemState.name);

			switch (state.systemState) {
				case state.systemStates.disabled:
					ui.updateUIElements("icons");
					ui.deactivate();
					break;
				case state.systemStates.ready:
					ui.updateUIElements("icons");
					ui.activate();
					ui.initAddonBarAsker();
					
					break;
			}
			
			ui.updateUIElements("menu");
			ui.updateStatusLabels();
			
		} catch (e){
			info.error(e);
		}
	},
	
	updateStatusLabels : function(win) {
		var info = new Info("ui.updateStatusLabels");
		
		try {
			if (!win) {
				ui.safeWindowIterator(function(iwin) {
					ui.updateStatusLabels(iwin);
				});
			} else {
				var statusLabel = win.document.getElementById('anonymox-statusLabel');
				var panel_profileLabel = win.document.getElementById('anonymox-infopanel-profilelabel');
				var panel_serviceLabel = win.document.getElementById('anonymox-infopanel-servicelabel');
				
				var statusColor;
				switch(state.systemState) {
					case state.systemStates.loading:
						statusColor = "orange";
						break;
					case state.systemStates.disabled:
						statusColor = "red";
						break;
				}
				
				if (state.systemState == state.systemStates.ready) {
					info.verbose("setting profile-based status");
					
					var profile = win.anonymoxdata.profile;

					if (profile.isInternal) {
						statusColor = "blue";
					} else if (profile.gateway != profilemgr.nullGateway) {
						statusColor = "green";
					} else {
						statusColor = "red";
					}
					
					if (profile.name == "default") {
						statusLabel.value = profile.gateway.id;
					} else {
						statusLabel.value = profile.getDisplayName() + " | " + profile.gateway.id;
					}
					
					panel_profileLabel.value = profile.getDisplayName();
					panel_serviceLabel.value = profile.gateway.id ? profile.gateway.id : "null";
					ui.updateUIElements("statusBarEntry", win);
				} else {
					info.verbose("setting status from global");
					
					statusLabel.value = state.systemState.tString;
					panel_profileLabel.value = state.systemState.tString;
					panel_serviceLabel.value = "-";
				}
				
				
				statusLabel.style.color = statusColor;
				
				panel_profileLabel.style.color = statusColor;
				panel_serviceLabel.style.color = statusColor;
			}
		} catch (e) {
			info.error(e);
		}
	},
	
	
	updateUIElements : function(key, win) {
		var info = new Info("ui.updateUIElements");

		if (!win) {
			ui.safeWindowIterator(function(iwin){
				ui.updateUIElements(key, iwin);
			});
		} else {
			var doc = win.document,
				statusBarIcon = doc.getElementById('anonymox-statusBarIcon');

			if (key == "icons" || !key) {
				if (doc.getElementById('anonymox-button'))
					doc.getElementById('anonymox-button').disabled = !state.isReady;
				
				ui.updatePremiumStatusIcon(core.isPremiumAccount);

				if (core.isPremiumAccount && state.isReady) {
					statusBarIcon.src = "chrome://anonymox/skin/icon-premium16.png";
				} else if (state.isReady) {
					statusBarIcon.src = "chrome://anonymox/skin/icon16.png";
				} else {
					statusBarIcon.src = "chrome://anonymox/skin/icon-disabled16.png";
				}
			}

			if (key == "menu" || !key){
				
				doc.getElementById("anonymox-mainMenu-toggleReferrer").setAttribute("checked",
						config.allPrefs.getIntPref("network.http.sendRefererHeader") == 0);
			
				
				var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo),
					platformVer = parseInt(appInfo.platformVersion, 10);

				if (platformVer < 22) {
					doc.getElementById("anonymox-mainMenu-clearcookies-3").style.display = "none";
				}
				

				
				doc.getElementById("anonymox-mainMenu-clearcookies-" +
						config.allPrefs.getIntPref("network.cookie.cookieBehavior"))
						.setAttribute("checked", true);
	
				
				var active;
				try {
					active = config.allPrefs.getCharPref("general.useragent.override");
				} catch (e) {
					
					active = "[[DEFAULT]]";
				}
				var userAgentMenuNodes = doc.getElementById("anonymox-mainMenu-userAgentList");
				for ( var i = 0; i < userAgentMenuNodes.childNodes.length; i++) {
					if (userAgentMenuNodes.childNodes[i].value == active || userAgentMenuNodes.childNodes[i].value == "[[CUSTOM]]") {
						userAgentMenuNodes.childNodes[i].setAttribute("checked", true);
						break;
					}
				}

				
				doc.getElementById("anonymox-mainMenu-lists").setAttribute("checked", prefs.getBoolPref("displayProfileBar"));
			}
			
			if (key == "statusBarEntry"){
				var statusBarEntry = doc.getElementById('anonymox-statusBarEntry');
				statusBarEntry.hidden = !prefs.getBoolPref("displayInStatusBar");
			}
			
			if (key == "extInfo" || !key){
				
				var bar_iplabel = doc.getElementById('anonymox-externalIp');
				var panel_iplabel = doc.getElementById('anonymox-infopanel-iplabel');
				bar_iplabel.hidden = !prefs.getBoolPref("displayIP");
				panel_iplabel.hidden = !prefs.getBoolPref("displayIP");
				
				var bar_flag = doc.getElementById('anonymox-externalFlag');
				var panel_iplabel = doc.getElementById('anonymox-infopanel-flag');
				bar_flag.hidden = !prefs.getBoolPref("displayFlag");
				panel_iplabel.hidden = !prefs.getBoolPref("displayFlag");
			}
		}
	},

	updatePremiumStatusIcon: function (isActive) {
		var btnImg,
			panelImg,
			statusText;

		ui.safeWindowIterator(function (window) {
			btnImg = utils.jQuery(window.document.getElementById("anonymox-button"));
			panelImg = window.document.getElementById("anonymox-panel-premiumStatusLogo");
			statusText = utils.jQuery(window.document.getElementById("anonymox-panel-premiumStatusLink"));

			if (isActive) {
				btnImg.addClass("premium");
				panelImg.src = "chrome://anonymox/skin/icon-premium24.png";
				statusText.attr("value", "Premium").removeClass("text-link");
			} else {
				btnImg.removeClass("premium");
				panelImg.src = "chrome://anonymox/skin/icon24.png";
				statusText.attr("value", "Free").attr("href", config.premiumInfoURI + "panelicon").addClass("text-link");
			}
		});
	},

	
	notification : function(level, messageid, messageargs, buttons, onlyCurrentTab) {
		let info = new Info("notification");
		
		let notificationPrefix = "showNotifications";
		
		
		if (level == 0) {
			if (prefs.getPrefType(notificationPrefix + "." + messageid) == prefs.PREF_INVALID) {
				info.error("pref " + notificationPrefix + "." + messageid + " not found");
			} else if (!prefs.getBoolPref(notificationPrefix + "." + messageid)) {
				info.debug("messageid: " + messageid + " not displayed per settings");
				return;
			}
		}
		
		let notificationID = messageid;
		
		if (!ui.notifications[notificationID]) {
			let debugMessage = "adding notification: " + notificationID;
			
			let buttonsArray = [];
			
			if (Array_isArray(buttons)) {
				buttonsArray = buttons;
			} else if (buttons) {
				buttonsArray = [buttons];
			}
			
			
			if (level == 0) {
				debugMessage += " with hide option";
				buttonsArray.push(
					{
						label: localize("neverShowAgain"),
						accessKey: "",
						callback: function() {
							prefs.setBoolPref(notificationPrefix + "." + messageid, false);
						}
					}
				);
			}
			
			var currentNotification = {
				label : localize(messageid, messageargs),
				value : notificationID,
				icon : 'chrome://anonymox/skin/icon16.png',
				priority : "PRIORITY_INFO_MEDIUM",
				buttons : buttonsArray,
				global : (onlyCurrentTab) ? false : true
			};
			
			ui.notifications[notificationID] = currentNotification;
			
			if (onlyCurrentTab) {
				debugMessage += " to current tab";
				info.debug(debugMessage);
				ui.addNotificationForWindow(onlyCurrentTab, currentNotification);
			} else {
				debugMessage += " to all tabs";
				info.debug(debugMessage);
				
				ui.safeWindowIterator(function (window) {
					ui.addNotificationForWindow(window, currentNotification);
				});
			}
		} else {
			info.debug("notification already pending: " + notificationID);
		}
	},
	
	addNotificationForWindow : function(window, notification) {
		let info = new Info("addNotificationForWindow");
		
		let notifyBox = window.document.getElementById("anonymox-notifications");
		if (notifyBox) {
			notifyBox.appendNotification(
				notification.label,
				notification.value,
				notification.icon,
				notifyBox[notification.priority],
				notification.buttons,
				ui.notificationCallbackMaker(notification.value)
			);
		} else {
			info.debug("notifyBox not found");
		}
	},
	
	removeNotificationForAllWindows : function (value) {
		if (ui.notifications[value]) {
			delete ui.notifications[value];
			ui.safeWindowIterator(function (window) {
				ui.removeNotificationForWindow(window, value);
			});
		}
	},
	
	notificationCallbackMaker : function (value) {
		return function (type) {
			if (ui.notifications[value]) {
				switch (type) {
					case "removed":
						delete ui.notifications[value];
						ui.safeWindowIterator(function (window) {
							ui.removeNotificationForWindow(window, value);
						});
				}
			}
		};
	},
	
	removeNotificationForWindow : function(window, value) {
		let info = new Info("removeNotificationForWindow");
		
		let notifyBox = window.document.getElementById("anonymox-notifications");
		
		if (notifyBox) {
			let notification = notifyBox.getNotificationWithValue(value);
			if (notification) {
				notifyBox.removeNotification(
					notification
				);
			}
		} else {
			info.debug("notifyBox not found");
		}
	},
	
	removeNonPersistantNotificationsForAllWindows : function() {
		for (var n in ui.notifications) {
			let currentNotification = ui.notifications[n];
			if (!currentNotification.global) {
				ui.safeWindowIterator(function (window) {
					ui.removeNotificationForWindow(window, currentNotification.value);
				});
			}
		}
	},
	
	notifyModal : function(text){
		var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
		promptService.alert(null, "anonymoX", text);
	},
	
	
	
	toggleReferrer : function() {
		if (browserapi.isReferrerEnabled()){
			var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
			if (promptService.confirm(null, "anonymoX", localize("confirmDeactivateReferrer")))
				browserapi.setReferrer(false);
		}
		else
			browserapi.setReferrer(true);
		
		ui.updateUIElements("menu");
		info.sendMsg(utils.msgType.usage, "features", "toggleReferrer");
	},
	
	premiumHint : function(win, src){
		var domain = utils.getDomainPartsFromURI(win.anonymoxdata.uri);
		if (domain == null) return;
		domain = domain[0];
		
		ui.notification(1, "premiumRequired", undefined,
			{
				label: localize("premiumRequired_infoButton"),
				accessKey: "M",
				callback: function() {
					browserapi.openURI(config.premiumInfoURI + src, 0);
				}
			}
		);
	},
	
	setUserAgent : function(obj) {
		if (obj.value != "[[DEFAULT]]"){
			var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
			if (promptService.confirm(null, "anonymoX", localize("confirmChangeUA")))
				browserapi.setUserAgent(obj.value);
		}
		else
			browserapi.setUserAgent(undefined);
		
		ui.updateUIElements("menu");
	},
		
	refreshExternalInfo : function(win){
		let info = new Info("updateRequest");
		if (!state.isReady) return;
		core.getExternalInfo(win.anonymoxdata.profile.gateway, true);
		info.sendMsg(utils.msgType.usage, "features", "refreshExternalInfo");
	},
	
	
	
	clearCookiesAsker : function(win){
		var domain = utils.getDomainPartsFromURI(win.anonymoxdata.uri);
		if (domain == null) return;	
		domain = domain[0];
		
		ui.notification(1, "confirmClearCookies", undefined,
			[
				{
					label: localize("confirmClearCookies_onlyThisPage", domain),
					accessKey: "O",
					callback: function() {
						browserapi.clearCookies(domain, "menu");
					}
				},
				{
					label: localize("confirmClearCookies_allPages"),
					accessKey: "A",
					callback: function() {
						browserapi.clearCookies(undefined, "menu");
					}
				}
			]
		);
	},
	
	acceptCookies : function(win, val){
		browserapi.acceptCookies(val, "acceptCookiesDeleteQuestion");
		
		if (val == 1 || val == 2)
			ui.notification(0, "disableCookiesDeleteHint");
	},
	
	
	
	
	updateRequest : function(version){
		var info = new Info("updateRequest");
		
		setTimeout(function(){
			ui.notification(1, "updateInstallQ", undefined,
				{
					label: localize("ok"),
					accessKey: "Y",
					callback: function() {
						Cu.import("resource://gre/modules/AddonManager.jsm");
						AddonManager.getInstallForURL("https://update.anonymox.net/anonymox-update-" + version + ".xpi", function(install){
							install.addListener({
								onInstallEnded : function(i,a){
									Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication).restart();
								},
								onInstallFailed : function(i){
									ui.notifyModal("Update failed. Please update manually from www.anonymox.net");
								},
								onDownloadFailed : function(i){
									ui.notifyModal("Update failed. Please update manually from www.anonymox.net");
								}
							});
							install.install();
						}, "application/x-xpinstall");
					}
				}
			);
		}, 2000);
	},
	
	
	initAddonBarAsker : function(){
		var info = new Info("initAddonBarAsker");
		try{
			if (prefs.getIntPref("addonBar") == -1) return;

			var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
			var win = wm.getMostRecentWindow("navigator:browser");
			if (!win) return;	
			
			var addonBar = win.document.getElementById("addon-bar");
			if (!addonBar){		
				ui.IABA_tryAgain_TO = setTimeout(function(){ui.initAddonBarAsker();}, 500);
				return;
			}
			
			
			if (!addonBar.collapsed && prefs.getIntPref("addonBar") != 1)
				prefs.setIntPref("addonBar", -1);
			
			else if (!ui.initAddonBarAskerTO){	
				ui.initAddonBarAskerTO = setTimeout(function()
				{
					ui.notification(1, "showAddonBarQuestion", undefined,
						[
							{
								label: localize("yes"),
								accessKey: "Y",
								callback: function() {
									ui.safeWindowIterator(function(iwin) {
										iwin.document.getElementById("addon-bar").collapsed = false;
									});
									prefs.setIntPref("addonBar", -1);
									
									var addonBar = win.document.getElementById("addon-bar");
									win.setToolbarVisibility(addonBar, true);
								}
							},
							{
								label: localize("noDontAskAnymore"),
								accessKey: "N",
								callback: function() {
									prefs.setIntPref("addonBar", -1);
								}
							}
						]
					);
					ui.initAddonBarAskerTO = undefined;
				}, 1000 * 6);
			}
		}catch(e){
			info.error(e);
		}
	},
	
	initUsageCollector : function(){
		var info = new Info("initUsageCollector");
		try{
			if (prefs.prefHasUserValue("usagecollection")){
				var prefVal = prefs.getIntPref("usagecollection");
				if (prefVal != 0 && prefVal != 1){	
					if (new Date().getTime() / 1000 > prefVal){
						
						if (!ui.initUsageCollectorTO){	
							ui.initUsageCollectorTO = setTimeout(function()
							{
								ui.notification(1, "usageCollectionQuestion", undefined,
									[
										{
											label: localize("yes"),
											accessKey: "Y",
											callback: function() {
												prefs.setIntPref("usagecollection", 1);
											}
										},
										{
											label: localize("noDontAskAnymore"),
											accessKey: "N",
											callback: function() {
												prefs.setIntPref("usagecollection", 0);
											}
										}
									]
								);
								ui.initUsageCollectorTO = undefined;
							},
							1000 * 5
							);
						}
					}
				}
			}else{
				prefs.setIntPref("usagecollection", new Date().getTime() / 1000 + 60 * 60 * config.usageCollectionIntervalHrs);
			}
		}catch(e){
			info.error(e);
		}
	}
};