/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";

var utils = {
		
	init : function(tmpWindow) {
		var info = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
		utils.versionInfo = "anonymoX " + config.version + " @ " + info.name + " " + info.version + " @ " + config.osname;
		
		
		var subscriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
		subscriptLoader.loadSubScript("resource://anonymox/external/jquery-1.7.1.js",tmpWindow);
		utils.jQuery = tmpWindow.jQuery.noConflict(true);
	},
	
	msgType : {
		usage : "usage",
		notice : "notice",
		error : "error"
	},

	
	
	localize : function(msg, args) {
		var info = new Info("translate");
		try {
			var stringBundle = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService).createBundle(
			"chrome://anonymox/locale/overlay.properties");
			
			if (args == undefined)
				return stringBundle.GetStringFromName(msg);
			else if (Array_isArray(args))
				return stringBundle.formatStringFromName(msg, args, args.length);
			else
				return stringBundle.formatStringFromName(msg, [args], 1);
		} catch(e) {
			info.error(e, "Error at getting string, id '" + msg + "'");
		}
	},
	
	displayFuncOrUndefined : function(func){
		return (func != undefined)?"<func>":undefined;
	},
	
	escapeForRegex : function(input){
		return input.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&");
	},
	
	getDomainPartsFromURI : function(uri){
		if (!uri) {throw "getDomainPartsFromURI: uri is undefined"};
		
		var matches = uri.match(/^(?:https?|ftp):\/\/([^:\/]*)/);
		if (matches != null) {
			var domain = matches[1],
				ret = new Array();
			
			if (domain.match(/\.\D+$/)){	
				
				let shortdomain = domain.match(/^www\.(.+)/);
				if (shortdomain) domain = shortdomain[1];

				var eTLDService = Components.classes["@mozilla.org/network/effective-tld-service;1"]
				.getService(Components.interfaces.nsIEffectiveTLDService);

				try {
					ret.push(eTLDService.getPublicSuffixFromHost(domain));
					
					
					try {
						var n = 0;
						while (true) {
							ret.push(eTLDService.getBaseDomainFromHost(domain, n++));
						}
					} catch(e) {}
					ret.reverse();
				} catch(e) {
					ret.push(domain);
				}
				return ret;
			} else {						
				ret.push(domain);
				ret.push("[ipv4]");	
				return ret;
			}
		} else
			return null;
	}
};

var Info = function(functionName){
	this.debugfilter = /.*/;
	this.verbosefilter = /^$/;
	
	this.verbose = function(msg){
		if (config.debugout && functionName.match(this.verbosefilter))
			this.genericLog("Verbose", msg);
	};
	this.debug = function(msg){
		if (config.debugout && functionName.match(this.debugfilter))
			this.genericLog("Debug", msg);
	};
	this.notice = function(msg){
		if (config.debugout)
			this.genericLog("Notice", msg);
		this.jsConsoleLog("Notice", msg);
	};
	this.warning = function(msg){
		this.genericLog("Warning", msg);
		this.jsConsoleLog("Warning", msg);
	};

	this.genericLog = function(severity, msg){
		dump("anonymox: " + severity + " [" + functionName + "] " + msg + "\n");
	};
	
	this.jsConsoleLog = function(severity, msg){
		var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].
			getService(Components.interfaces.nsIConsoleService);
		aConsoleService.logStringMessage("anonymox: " + severity + " [" + functionName + "] " + msg + "\n");
	};
	this.logObject = function(obj, fullLog){
		this.debug("logObject : " + obj);
		for (property in obj) {
			var out = obj[property];
			if (!fullLog && out!=null && out.toString().length > 20)
				out = out.toString().substring(0,20) + "[....]";
			this.debug("---- " + property + " = " + out);
		}
	};
	this.serializeArray = function(ar, dispFunct){
		var out = "[";
		if (ar)
			for ( var i = 0; i < ar.length; i++) {
				out += (!dispFunct ? ar[i] : dispFunct(ar[i])) + (i!=ar.length-1 ? " " : "");
			}
		return out + "]";
	};
	this.serializeObject = function(obj, excludeRegEx){
		var o = "";
		for (property in obj) {
			if (excludeRegEx && excludeRegEx.match(property))
				continue;
			var add = obj[property];
			if (add != null && add.toString().length > 20)
				add = add.toString().substring(0, 20) + "[....]";
			o += property + "=" + add + " ";
		}
		return o;
	};
	this.error = function(errorObj, additionalInfo){
		var errMsg = "Source: " + functionName + "\nError Message: " + errorObj + (additionalInfo ? "\nAdd. Info: " + additionalInfo : "") + "\nCode: " + errorObj.fileName + " line " + errorObj.lineNumber + "\nVersion: " + utils.versionInfo;
		
		this.genericLog("Error", "-----anonymoX Fatal Error-----\n\n" + errMsg);
		this.jsConsoleLog("Error", "-----anonymoX Fatal Error-----\n\n" + errMsg);
		
		if (!core){	
			var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
			promptService.alert(null, "anonymoX", errMsg);
			return;
		}
		
		if (state.offDueToFailure)
			this.debug("aborting handling of the following error because already hit error: " + errorObj);
		else{
			
			var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);
			
			state.offDueToFailure = true;
			core.switchOff();
			
			if (promptService.confirm(null, "anonymoX Error", localize("error") + "\n" + errMsg))
				browserapi.openURI(config.helpErrorPageUri, 0);
			
			this.sendMsg(utils.msgType.error, functionName, errMsg);
		}
	};
	this.sendMsg = function(type, section, content, includeUID) {	
		this.verbose("sendMsg[delayed]: type: " + type + " section: " + section + " content: " + content + " includeUID: " + includeUID);
		setTimeout(function(){
			let includeVersions = type == utils.msgType.error || type == utils.msgType.notice;
			if (type != utils.msgType.usage || prefs.prefHasUserValue("usagecollection") && prefs.getIntPref("usagecollection") == 1){
				communication.csclient.Log(type, section, content, includeUID ? config.user : undefined, includeVersions?utils.versionInfo:undefined, function(){});
			}
		},10);
	};
};


var Array_remove = function(ar, from, to) {
	var rest = ar.slice((to || from) + 1 || ar.length);
	ar.length = from < 0 ? ar.length + from : from;
	return ar.push.apply(ar, rest);
};
var Array_inArray = function(ar, needle) {
	if (!ar) return false;
	for ( var i = 0; i < ar.length; i++)
		if (ar[i] === needle)
			return true;
	return false;
};
var Array_equal = function(array1, array2) {
	if (!array1 && !array2)
		return true;
	if (!array1 ^ !array2)
		return false;
	if (array1.length != array2.length)
		return false;
	
	for ( var i = 0; i < array1.length; i++)
		if (array1[i] != array2[i])
			return false;
	return true;
};
var Array_isArray = function(obj) {
	return Object.prototype.toString.call(obj) === "[object Array]";
};

var setTimeout = function(func, delay) {
	var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
	var callback = {
		notify : function() {
			func.call();
		}
	};
	timer.initWithCallback(callback, delay, Ci.nsITimer.TYPE_ONE_SHOT);
	return timer;
};
var clearTimeout = function(timer) {
	timer.cancel();
};


var localize = utils.localize;

